package net.conacom.servletrunner.impl;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.util.*;
import java.text.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;
import net.conacom.servletrunner.util.*;
import net.conacom.servletrunner.main.*;
/**
 * Extends the {@link javax.servlet.ServletRequest} interface
 * to provide additional functionality for the request object
 * that is passed to an HTTP servlet. 
 *
 * <p>The servlet engine implements
 * this interface to create <code>HttpServletRequest</code> objects,
 * which pass information from the client to the <code>service<code>
 * method of an <code>HttpServlet</code>.
 *
 *
 * @author 	Various
 * @version	$Version$
 */
/**
 * 25.06.02 ho: API 2.3
 * 24.06.02 ee: API 2.2, several fixes in the Session behaviour (EE)
 * 13.05.02 ho: platformpath for getPathTranslated()
 * 08.04.02 ho: fixed session-support
 * 29.03.02 ho: url-support for sessions
 * 21.03.02 ho: creation
 */
public class HttpServletRequest_Impl extends ServletRequest_Impl implements HttpServletRequest {
	// a HttpSession created in the HttpServletRequest
	private HttpSession_Impl sessLocalSession = null;
	// the servlet context (needed for HttpSession
	private ServletContext srvlContext = null;
public HttpServletRequest_Impl(ServletContainerConfig scc, ServletInstance si) {
	super(scc, si);
}
/**
 * 
 */
public void dispose() {
	super.dispose();
	if (sessLocalSession != null) {
		sessLocalSession.dispose();
		sessLocalSession = null;
	}
}
/**
 * Returns the name of the authentication scheme the
 * server uses, for example, "BASIC" or "SSL," or <code>null</code>
 * if the server does not have an authentication scheme. 
 *
 * <p>The authentication scheme provides a challenge-response
 * model in which the server challenges the client,
 * and the client provides authentication information.
 * Same as the value of the CGI variable AUTH_TYPE.
 *
 *
 * @return		a <code>String</code> specifying the name of
 *			the authentication scheme, or
 *			<code>null</code> if the server
 *			does not have an authentication
 *			scheme
 */
public String getAuthType() {
	return getCGI_Environment0("AUTH_TYPE");
}
/**
 * Returns the portion of the request URI that indicates the context of the request. The context 
 * path always comes first in a request URI. The path starts with a "/" character but does not end 
 * with a "/" character. For servlets in the default (root) context, this method returns "". The 
 * container does not decode this string.
 *
 * Returns: a String specifying the portion of the request URI that indicates the context of the request
 */
/**
 * ho: this was filled when creating the container
 */
public String getContextPath() {
	return (String) getAttribute(Runner.ATTR_PRAEFIX + "servletidentifier");
}
/**
 * Returns an array containing all of the <code>Cookie</code>
 * objects the browser sent with this request.
 * This method returns <code>null</code> if the browser did 
 * not send any cookies.
 *
 * @return		an array of all the <code>Cookies</code>
 *			included with this request, or <code>null</code>
 *			if the request has no cookies
 *
 * ho: this is almost Ewans implementation from the iMatix servlet runner
 */
public Cookie[] getCookies() {
	Vector vecCookies = new Vector();
	String strCookieList = getCGI_Environment0("HTTP_COOKIE");
	int intCookieVersion = 0;
	if (strCookieList != null) {
		// Parse cookies string, looking for name=value pairs.  If it is
		// RFC2109 then the first will be $Version, and any other
		// $... names will be special.  We save the version for
		// later, and set on each cookie (if non-zero).
		// Cookies are separated by either ";" or "," (RFC2109 says
		// ";" separates attributes of cookie, "," separates cookie).
		//
		// XXX: We need to handle ";" and "," inside the quotes as
		// being part of the attribute/value; currently parsing will
		// go wrong.
		//
		// XXX: Should parse cookies once and then keep array around
		// to pass out/clone.
		//
		// XXX: HTTP_COOKIE2 appears to contain version string, if
		// any.
		//
		final int attributeSep = (int) ';';
		final int cookieSep = (int) ',';
		final int equals = (int) '=';
		final int length = strCookieList.length();
		int start = 0;
		while (start < length) {
			// Get Name of pair
			int end = strCookieList.indexOf(equals, start);
			String name = strCookieList.substring(start, end);
			start = end + 1;

			// Find value of pair
			int nextAttSep = strCookieList.indexOf(attributeSep, start);
			int nextCookieSep = strCookieList.indexOf(cookieSep, start);
			if (nextCookieSep > start && nextCookieSep < nextAttSep)
				end = nextCookieSep;
			else
				end = nextAttSep;
			if (end < 0)
				end = length;
			String value = strCookieList.substring(start, end);
			start = end + 1;

			// Tidy up the name that we retrieved
			name = name.trim();

			// Tidy up the value that we retrieved (trim off
			// separator, strip double quotes)
			//
			value = value.trim();
			if (value.endsWith(";") || value.endsWith(","))
				value = value.substring(0, value.length() - 1);
			if (value.startsWith("\"") && value.endsWith("\""))
				value = value.substring(1, value.length() - 1);

			// Figure out if this is RFC2109 attribute ($....)
			if (name.startsWith("$")) {
				if (name.equalsIgnoreCase("$Version")) {
					try {
						intCookieVersion = Integer.parseInt(value);
					} catch (NumberFormatException e) {
					}
				} else
					if (name.equalsIgnoreCase("$Path")) {
						Cookie last = (Cookie) vecCookies.lastElement();
						if (last != null)
							last.setPath(value);
					} else
						if (name.equalsIgnoreCase("$Domain")) {
							Cookie last = (Cookie) vecCookies.lastElement();
							if (last != null)
								last.setDomain(value);
						}
			} else {
				// Nope, it is a new cookie.  Let's create one.
				Cookie cookie = new Cookie(name, value);
				if (intCookieVersion > 0)
					cookie.setVersion(intCookieVersion);
				vecCookies.addElement(cookie);
			}
		}
	}

	// Pull cookies out into an array
	if (vecCookies.size() == 0)
		return null;
	Cookie c[] = new Cookie[vecCookies.size()];
	vecCookies.copyInto(c);
	return c;
}
/**
 * Returns the value of the specified request header
 * as a <code>long</code> value that represents a 
 * <code>Date</code> object. Use this method with
 * headers that contain dates, such as
 * <code>If-Modified-Since</code>. 
 *
 * <p>The date is returned as
 * the number of milliseconds since January 1, 1970 GMT.
 * The header name is case insensitive.
 *
 * <p>If the request did not have a header of the
 * specified name, this method returns -1. If the header
 * can't be converted to a date, the method returns
 * an <code>IllegalArgumentException</code>.
 *
 * @param name		a <code>String</code> specifying the
 *				name of the header
 *
 * @return			a <code>long</code> value
 *				representing the date specified
 *				in the header expressed as
 *				the number of milliseconds
 *				since January 1, 1970 GMT,
 *				or -1 if the named header
 *				was not included with the
 *				reqest
 */
public long getDateHeader(String strKey) {
	String strTmp = getHeader(strKey);
	if (strTmp == null)
		return -1l;
	try {
		return DateFormat.getDateInstance().parse(strTmp).getTime();
	} catch (ParseException e) {
		throw new IllegalArgumentException("Field does not contain date");
	}
}
/**
 * Returns the value of the specified request header
 * as a <code>String</code>. If the named header wasn't 
 * sent with
 * the request, this method returns <code>null</code>.
 * The header name is case insensitive. You can use
 * this method with any request header.
 *
 * @param name	a <code>String</code> specifying the
 *				header name
 *
 * @return		a <code>String</code> containing the
 *				value of the requested
 *				header, or <code>null</code>
 *				if the request does not
 *				have a header of that name
 */
public String getHeader(String strKey) {
	return getCGI_Environment0("HTTP_" + strKey.toUpperCase());
}
/**
 * Returns an enumeration of all the header names
 * this request contains. If the request has no
 * headers, this method returns an empty enumeration.
 *
 * <p>Some servlet engines do not allow do not allow
 * servlets to access headers using this method, in
 * which case this method returns <code>null</code>
 *
 * @return		an enumeration of all the
 *				header names sent with this
 *				request; if the request has
 *				no headers, an empty enumeration;
 *				if the servlet engine does not
 *				allow servlets to use this method,
 */
public Enumeration getHeaderNames() {
	Vector vec = new Vector();
	for (Enumeration e = hshHeader.keys(); e.hasMoreElements();) {
		String s = (String) e.nextElement();
		if (s.startsWith("HTTP_")) {
			s = s.substring(5);
			vec.addElement(s);
			if (blnMixedHeader)
				vec.addElement(Helper.cgikey2headerkey(s));
		}
	}
	return vec.elements();
}
/**
 * Returns all the values of the specified request header as an Enumeration of String objects.
 *
 * Some headers, such as Accept-Language can be sent by clients as several headers each with a 
 * different value rather than sending the header as a comma separated list.
 *
 * If the request did not include any headers of the specified name, this method returns an empty 
 * Enumeration. The header name is case insensitive. You can use this method with any request header.
 *
 * Parameters: 
 *   strKey - a String specifying the header name
 * 
 * Returns: 
 *   an Enumeration containing the values of the requested header. If the request 
 *   does not have any headers of that name return an empty enumeration. If the container 
 *   does not allow access to header information, return null
 */
public Enumeration getHeaders(String strKey) {
	return Helper.stringEnumerator(getCGI_Environment(strKey));
}
/**
 * Returns the value of the specified request header
 * as an integer. If the request does not have a header
 * of the specified name, this method returns -1. If the
 * header cannot be converted to an integer, this method
 * throws a <code>NumberFormatException</code>.
 *
 * <p>The header name is case insensitive.
 *
 * @param name	a <code>String</code> specifying the name
 *				of a request header
 *
 * @return		an integer expressing the value 
 * 				of the request header or -1
 *				if the request doesn't have a
 *				header of this name
 */
public int getIntHeader(String strKey) {
	String s = getHeader(strKey);
	return s == null ? -1 : Integer.parseInt(s);
}
/**
 * Returns the name of the HTTP method with which this 
 * request was made, for example, GET, POST, or PUT.
 * The returned <code>String</code> is the same as
 * the value of the CGI variable REQUEST_METHOD.
 *
 * @return			a <code>String</code> 
 *				specifying the name
 *				of the method with which
 *				this request was made
 */
public String getMethod() {
	return getCGI_Environment0("REQUEST_METHOD");
}
/**
 * Returns any extra path information associated with
 * the URL the client sent when it made this request.
 * The extra path information follows the servlet path
 * (the URI of the URL) but precedes the query string.
 * This method returns <code>null</code> if there
 * was no extra path information.
 *
 * <p>The information this method returns is
 * the same as the value of the CGI variable PATH_INFO.
 *
 *
 * @return	a <code>String</code> specifying 
 *			extra path information that comes
 *			after the servlet path but before
 *			the query string in the request URL;
 *			or <code>null</code> if the URL does not have
 *			any extra path information
 */
public String getPathInfo() {
	return Helper.platformPath(getCGI_Environment0("PATH_INFO"));
}
/**
 * Returns any extra path information after the servlet name
 * but before the query string, and translates it to a real
 * path. Same as the value of the CGI variable PATH_TRANSLATED.
 *
 * <p>If the URL does not have any extra path information,
 * this method returns <code>null</code>.
 *
 *
 * @return	a <code>String</code> specifying the
 *			real path, or <code>null</code> if
 *			the URL does not have any extra path
 *			information
 */
public String getPathTranslated() {
	return Helper.platformPath(getCGI_Environment0("PATH_TRANSLATED"));
}
/**
 * Returns the query string that is contained in the request
 * URL after the path. This method returns <code>null</code>
 * if the URL does not have a query string. Same as the value
 * of the CGI variable QUERY_STRING.
 *
 * @return	a <code>String</code> containing the query
 *			string or <code>null</code> if the URL 
 *			contains no query string
 */
public String getQueryString() {
	return getCGI_Environment0("QUERY_STRING");
}
/**
 * Returns the name of the user making this request, if the
 * user has logged in using HTTP authentication. This method
 * returns <code>null</code> if the user login is not authenticated.
 * Whether the user name is sent with each subsequent request
 * depends on the browser. Same as the value of the CGI variable
 * REMOTE_USER.
 *
 * @return		a <code>String</code> specifying the name
 *			of the user making this request, or <code>null</code
 *			if the user name is not known
 */
public String getRemoteUser() {
	return getCGI_Environment0("REMOTE_USER");
}
/**
 * Returns the session ID specified by the client. This may
 * not be the same as the ID of the actual session in use.
 * For example, if the request specified an old (expired)
 * session ID and the server has started a new session, this
 * method gets a new session with a new ID. If the request
 * did not specify a session ID, this method returns
 * <code>null</code>.
 *
 *
 * @return		a <code>String</code> specifying the session
 *			ID, or <code>null</code> if the request did
 *			not specify a session ID
 *
 * @see		#isRequestedSessionIdValid
 */
public String getRequestedSessionId() {
	// we use the setting from the setup.
	if (scc.blnUseCookieForSession)
		return getSessionIDFromCookie();
	else
		return getSessionIDFromURL();
}
/**
 * Returns the part of this request's URL from the protocol
 * name up to the query string in the first line of the HTTP request.
 * For example:
 *
 * <blockquote>
 * <table>
 * <tr align=left><th>First line of HTTP request<th>
 * <th>Returned Value
 * <tr><td>POST /some/path.html HTTP/1.1<td><td>/some/path.html
 * <tr><td>GET http://foo.bar/a.html HTTP/1.0
 * <td><td>http://foo.bar/a.html
 * <tr><td>HEAD /xyz?a=b HTTP/1.1<td><td>/xyz
 * </table>
 * </blockquote>
 *
 * <p>To reconstruct an URL with a scheme and host, use
 * {@link HttpUtils#getRequestURL}, which
 * returns a <code>StringBuffer</code>.
 *
 * @return	a <code>String</code> containing
 *			the part of the URL from the 
 *			protocol name up to the query string
 *
 * @see		HttpUtils#getRequestURL
 */
public String getRequestURI() {
	String strServletPath = getServletPath();
	String strPathInfo = getPathInfo();
	if (strServletPath == null)
		return strPathInfo;
	else
		return strServletPath;
}
/**
 * Reconstructs the URL the client used to make the request. The returned URL contains 
 * a protocol, server name, port number, and server path, but it does not include query 
 * string parameters.
 *
 * Because this method returns a StringBuffer, not a string, you can modify the URL easily, 
 * for example, to append query parameters.
 *
 * This method is useful for creating redirect messages and for reporting errors.
 *
 * Returns: a StringBuffer object containing the reconstructed URL
 */
public StringBuffer getRequestURL() {
	return HttpUtils.getRequestURL(this);
}
/**
 * Returns the part of this request's URL that calls
 * the servlet. This includes either the servlet name or
 * a path to the servlet, but does not include any extra
 * path information or a query string. Same as the value 
 * of the CGI variable SCRIPT_NAME.
 *
 *
 * @return		a <code>String</code> containing
 *			the name or path of the servlet being
 *			called, as it is specified in
 *			the request URL 
 */
public String getServletPath() {
	String s = getCGI_Environment0("SCRIPT_NAME");
	if (s.endsWith("/"))
		s = s.substring(0, s.length() - 1);
	return s;
}
/**
 * Returns the current session associated with this request,
 * or if the request does not have a session, creates one.
 * 
 * @return		the <code>HttpSession</code> associated
 *			with this request
 */
public HttpSession getSession() {
	return getSession(true);
}
/**
 * Returns the current <code>HttpSession</code>
 * associated with this request or, if necessary, creates
 * a new session for the request. Use <code>true</code>
 * for <code>create</code> to create a new session,
 * or <code>false</code> to return the current
 * <code>HttpSession</code>. 
 *
 * <p>If <code>create</code> is <code>false</code>
 * and the request has no valid <code>HttpSession</code>,
 * this method returns <code>null</code>.
 *
 * <p>To make sure the session is properly maintained,
 * you must call this method at least once before 
 * you write any output to the response. Newly created 
 * sessions (that is, sessions for which 
 * <code>HttpSession.isNew</code> returns <code>true</code>)
 * do not have any application-specific state.
 *
 *
 *
 *
 * @param	<code>true</code> to create
 *			a new session for this request; 
 *			<code>false</code> to return the current
 *			session
 *			
 *
 * @return 	the <code>HttpSession</code> associated 
 *			with this request or <code>null</code> if
 * 			<code>create</code> is <code>false</code>
 *			and the request has no valid session
 *
 */
public HttpSession getSession(boolean blnCreate) {
	/**
	 * check if session is valid
	 */
	boolean blnValidSession = isRequestedSessionIdValid();
	if (!blnValidSession && !blnCreate) {
		return null;
	}

	/**
	 * we can return an existing session ?
	 */
	if (blnValidSession) {
		// session is in the context ? return it.
		if (sessLocalSession != null) {
			sessLocalSession._setServletContext(srvlContext);
			return sessLocalSession;
		}
		// the session-ID (from URL or cookie)
		String strID = getRequestedSessionId();
		sessLocalSession = scc.smManager.getHttpSession(strID);
		sessLocalSession._setServletContext(srvlContext);
		return sessLocalSession;
	}

	/**
	 * we shall create a session ?
	 */
	if (blnCreate) {
		sessLocalSession = scc.smManager.createHttpSession(this);
		sessLocalSession._setServletContext(srvlContext);
		return sessLocalSession;
	}
	// well ....
	return null;
}
/**
 * returns the session-id from cookie SESSION_IDENTIFIER or NULL if no ID was found.
 */
private String getSessionIDFromCookie() {
	if (!scc.blnUseCookieForSession)
		return null;
	//
	if (sessLocalSession != null) {
		return sessLocalSession.getId();
	}
	//
	Cookie cookies[] = getCookies();
	if (cookies != null) {
		for (int i = 0; i < cookies.length; i++)
			if (cookies[i].getName().equals(HttpSession_Impl.SESSION_IDENTIFIER))
				return cookies[i].getValue();
	}
	// If we fall through to here no cookie requesting session
	return null;
}
/**
 * returns the session-id from parameter SESSION_IDENTIFIER or NULL if no ID was found.
 */
private String getSessionIDFromURL() {
	if (scc.blnUseCookieForSession)
		return null;
	if (sessLocalSession != null) {
		return sessLocalSession.getId();
	}
	return getParameter(HttpSession_Impl.SESSION_IDENTIFIER);
}
/**
 * Returns a java.security.Principal object containing the name of the current authenticated user. 
 * If the user has not been authenticated, the method returns null.
 *
 * Returns:
 *   a java.security.Principal containing the name of the user making this request; null if the 
 *   user has not been authenticated
 */
public Principal getUserPrincipal() {
	if (getAuthType() == null || getAuthType().equals(""))
		return null;
	String strRemoteName = getCGI_Environment0("REMOTE_USER");
	if (strRemoteName == null || strRemoteName.equals(""))
		return null;
	return new sun.security.acl.PrincipalImpl(strRemoteName);
}
/**
 * Checks whether the session ID this request submitted
 * came in as a cookie, rather than from the {@link #getSession}
 * method.
 *
 * @return			<code>true</code> if the session ID
 *				came in as a
 *				cookie; otherwise, <code>false</code>
 *
 *
 * @see			#getSession
 */
public boolean isRequestedSessionIdFromCookie() {
	return getSessionIDFromCookie() != null;
}
/**
 * Checks whether the session ID this request submitted
 * came in as part of the request URL, rather than
 * from the {@link #getSession} method.
 *
 * @return		<code>true</code> if the session ID
 *				came in as part of an URL; otherwise,
 *				<code>false</code>
 *
 *
 * @see			#getSession
 */
public boolean isRequestedSessionIdFromURL() {
	return getSessionIDFromURL() != null;
}
/**
 * @deprecated	As of Version 2.1 of the Java Servlet
 *				API, use {@link #isRequestedSessionIdFromURL}
 *				instead.
 *
 */
public boolean isRequestedSessionIdFromUrl() {
	return isRequestedSessionIdFromURL();
}
/**
 * Checks whether this request has a valid session in the
 * current session context (which is a
 * {@link HttpSessionContext}). 
 * If the session is not valid,
 * the {@link #getSession} method never returns it.
 *
 * @return			<code>true</code> if this
 *				request has a valid session
 *				in the current session context;
 *				otherwise, <code>false</code>
 *
 * @see			#getRequestedSessionId
 * @see			#getSession
 * @see			HttpSessionContext
 */
public boolean isRequestedSessionIdValid() {
	// get the ID from URL or Cookie
	String strID = getRequestedSessionId();
	/**
	 * we're calling the sessionmanager directly because it may have a faster method to
	 * validate the session instead of retrieving the whole session information (hashtable et al)
	 */
	return scc.smManager.isValid(strID);
}
/**
 * Returns a boolean indicating whether this request was made using a secure channel, such as HTTPS.
 *
 * Returns:
 *   a boolean indicating if the request was made using a secure channel
 */
/**
 * ho: untested
 */
public boolean isUserInRole(String strRole) {
	Principal pp = getUserPrincipal();
	return pp.getName().equalsIgnoreCase(strRole);
}
/**
 * HELPER: returns the used or created HttpSession by this request to store it after leaving .service(...)
 * in ServletInstance 
 */
public HttpSession_Impl _getHttpSession() {
	//EE: We  need to check session is valid or not
	if (sessLocalSession != null && sessLocalSession.getLastAccessedTime() == -1)
		return null;
	else
		return sessLocalSession;
}
/**
 * initializes this object
 */
public void _initObject(Hashtable hshHeader, Hashtable hshParams, ServletInputStream sisRequest) {
	super._initObject(hshHeader, hshParams, sisRequest);
	if (sessLocalSession != null) {
		sessLocalSession.dispose();
		sessLocalSession = null;
	}
}
}