package net.conacom.servletrunner.impl;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import net.conacom.servletrunner.main.*;
import net.conacom.servletrunner.util.*;
/**
 * Defines an HTTP servlet response that a servlet running on a
 * Web server sends to a client using HTTP. This interface
 * allows the servlet's <code>service</code> method to access
 * HTTP headers and return data to its client. The servlet engine
 * implements this interface.
 *
 * 
 * @author	Various
 * @version	$Version$
 *
 * @see		javax.servlet.ServletResponse
 */
/**
 * 25.06.02 ho: API 2.3
 * 24.06.02 ee: API 2.2
 * 07.06.02 ho: encodeRedirectURL(..) checks for hostname. _getStatus*()
 * 31.05.02 ho: minor fix in creating the redirectURL
 * 28.04.02 ho: fixed headerbug. insert version when Content-Type starts with "text/"
 * 08.04.02 ho: ServletContainerConfig as parameter
 * 07.04.02 ho: fixed bug in cookiesupport (header)
 * 29.03 02 ho: encodeURL(...), encodeRedirectURL(...)
 * 27.03.02 ho: garbled output by xitami when no headerfield was set (_getResonseData())
 * 21.03.02 ho: creation
 */
public class HttpServletResponse_Impl extends ServletResponse_Impl implements HttpServletResponse {
	// storage for containers
	private Vector vecCookies = null;
	// the HttpServletRequest_Impl where we can get the Session (encodeRedirectURL, encodeURL)
	private HttpServletRequest_Impl srvlRequest = null;
/**
 * 
 */
public HttpServletResponse_Impl(ServletContainerConfig scc) {
	super(scc);
	vecCookies = new Vector();
}
/**
 * Adds the specified cookie to the response.  It can be called
 * multiple times to set more than one cookie.
 *
 * @param cookie the Cookie to return to the client
 */
public void addCookie(Cookie cookie) {
	vecCookies.addElement(cookie);
}
/**
 * API 2.2
 * Adds a response header with the given name and date-value. The date is specified in terms 
 * of milliseconds since the epoch. This method allows response headers to have multiple values.
 *
 * Parameters:
 *   name - the name of the header to setvalue - the additional date value
 */
public void addDateHeader(String strKey, long lngDate) {
	addHeader(strKey, DateFormat.getDateInstance().format(new Date(lngDate)));
}
/**
 * API 2.2
 * Adds a response header with the given name and value. This method allows 
 * response headers to have multiple values.
 *
 * Parameters:
 *   name - the name of the headervalue - the additional header value 
 */
public void addHeader(String strKey, String strData) {
	if (strKey == null || strKey.equals("")) {
		throw new IllegalArgumentException("Wrong parameter: " + strKey);
	}
	dvHeaders.addKeyData(strKey, strData);
}
/**
 * API 2.2
 * Sets a response header with the given name and integer value. If the header 
 * had already been set, the new value overwrites the previous one. The containsHeader 
 * method can be used to test for the presence of a header before setting its value.
 *
 * Parameters:
 *    name - the name of the headervalue - the assigned integer value
 */
public void addIntHeader(String strKey, int intData) {
	addHeader(strKey, Integer.toString(intData));
}
/**
 * Checks whether the response message header has a field with
 * the specified name.
 * 
 * @param name the header field name
 * @return true if the response message header has a field with
 * the specified name; false otherwise
 */
public boolean containsHeader(String strKey) {
	return dvHeaders.containsKey(strKey);
}
/**
 * 
 */
public void dispose() {
	super.dispose();
	if (vecCookies != null) {
		vecCookies.removeAllElements();
		vecCookies = null;
	}
}
/**
 * Encodes the specified URL for use in the
 * <code>sendRedirect</code> method or, if encoding is not needed,
 * returns the URL unchanged.  The implementation of this method
 * should include the logic to determine whether the session ID
 * needs to be encoded in the URL.  Because the rules for making
 * this determination differ from those used to decide whether to
 * encode a normal link, this method is seperate from the
 * <code>encodeUrl</code> method.
 * 
 * <p>All URLs sent to the HttpServletResponse.sendRedirect
 * method should be run through this method.  Otherwise, URL
 * rewriting canont be used with browsers which do not support
 * cookies.
 *
 * <p>After this method is called, the response should be considered
 * to be committed and should not be written to.
 *
 * @param url the url to be encoded.
 * @return the encoded URL if encoding is needed; the unchanged URL
 * otherwise.
 *
 * @see #sendRedirect
 * @see #encodeUrl
 * @deprecated use encodeRedirectURL
 */
public String encodeRedirectUrl(String strURL) {
	return encodeRedirectURL(strURL);
}
/**
 * Encodes the specified URL for use in the
 * <code>sendRedirect</code> method or, if encoding is not needed,
 * returns the URL unchanged.  The implementation of this method
 * should include the logic to determine whether the session ID
 * needs to be encoded in the URL.  Because the rules for making
 * this determination differ from those used to decide whether to
 * encode a normal link, this method is seperate from the
 * <code>encodeUrl</code> method.
 * 
 * <p>All URLs sent to the HttpServletResponse.sendRedirect
 * method should be run through this method.  Otherwise, URL
 * rewriting canont be used with browsers which do not support
 * cookies.
 *
 * @param url the url to be encoded.
 * @return the encoded URL if encoding is needed; the unchanged URL
 * otherwise.
 */
public String encodeRedirectURL(String strURL) {

	// test if the redirect points to a protocol. in this case, don't encode session-id and servletname
	int pp1 = strURL.indexOf("?");
	int pp2 = strURL.indexOf("://");
	if ((pp2 < pp1) || (pp2 != -1 && pp1 == -1)) {
		return strURL;
	}

	// encode it
	return encodeURL(strURL);
}
/**
 * Encodes the specified URL by including the session ID in it,
 * or, if encoding is not needed, returns the URL unchanged.
 * The implementation of this method should include the logic to
 * determine whether the session ID needs to be encoded in the URL.
 * For example, if the browser supports cookies, or session
 * tracking is turned off, URL encoding is unnecessary.
 * 
 * <p>All URLs emitted by a Servlet should be run through this
 * method.  Otherwise, URL rewriting cannot be used with browsers
 * which do not support cookies.
 *
 * @param url the url to be encoded.
 * @return the encoded URL if encoding is needed; the unchanged URL
 * otherwise.
 */
public String encodeURL(String strURL) {
	//
	strURL = strURL.replace('\\', '/');
	// session is from cookie ? return unchanged
	if (scc.blnUseCookieForSession)
		return strURL;

	// there isn't any session created or retrieved yet ?
	HttpSession_Impl sess = srvlRequest._getHttpSession();
	if (sess == null) {
		return strURL;
	}
	// session isn't valid ?
	if (!sess._getSessionData().isValidSession()) {
		return strURL;
	}
	/**
	 * insert our session-ID
	 */
	// no other parameters ? add ID and return
	if (strURL.indexOf("?") == -1) {
		return strURL + "?" + HttpSession_Impl.SESSION_IDENTIFIER + "=" + srvlRequest._getHttpSession().getId();
	}
	// SESSION_IDENTIFIER already in the URL ? replace ID
	// XXX ho: UNTESTED
	int pp1 = strURL.indexOf(HttpSession_Impl.SESSION_IDENTIFIER);
	if (pp1 != -1) {
		pp1 = pp1 + HttpSession_Impl.SESSION_IDENTIFIER.length() + 1;
		int pp2 = Math.min(strURL.length(), strURL.indexOf("&", pp1));
		return strURL.substring(0, pp1) + srvlRequest._getHttpSession().getId() + strURL.substring(pp2);
	}
	// no session-ID found ? add
	return strURL + "&" + HttpSession_Impl.SESSION_IDENTIFIER + "=" + srvlRequest._getHttpSession().getId();
}
/**
 * Encodes the specified URL by including the session ID in it,
 * or, if encoding is not needed, returns the URL unchanged.
 * The implementation of this method should include the logic to
 * determine whether the session ID needs to be encoded in the URL.
 * For example, if the browser supports cookies, or session
 * tracking is turned off, URL encoding is unnecessary.
 * 
 * <p>All URLs emitted by a Servlet should be run through this
 * method.  Otherwise, URL rewriting cannot be used with browsers
 * which do not support cookies.
 *
 * @param url the url to be encoded.
 * @return the encoded URL if encoding is needed; the unchanged URL
 * otherwise.
 * @deprecated Use encodeURL(String url)
 */
public String encodeUrl(String strURL) {
	return encodeURL(strURL);
}
/**
 * Sends an error response to the client using the specified
 * status code and a default message.
 * @param sc the status code
 * @exception IOException If an I/O error has occurred.
 */
/**
 * ho: we may use the printwriter. but what happens, if sb. got the stream already ?
 */
public void sendError(int intStatus) throws IOException {
	bsResponse.append("<html>\n");
	bsResponse.append(" <head>\n");
	bsResponse.append("  <title>Error " + intStatus + "</title>\n");
	bsResponse.append(" </head>\n");
	bsResponse.append(" <body>\n");
	bsResponse.append("  An error " + intStatus + " has occurred<br>\n");
	bsResponse.append("  <hr>");
	bsResponse.append("[End of message]<br>\n");
	bsResponse.append(" </body>\n");
	bsResponse.append("</html>\n");
	setStatus(intStatus);
}
/**
 * Sends an error response to the client using the specified status
 * code and descriptive message.  If setStatus has previously been
 * called, it is reset to the error status code.  The message is
 * sent as the body of an HTML page, which is returned to the user
 * to describe the problem.  The page is sent with a default HTML
 * header; the message is enclosed in simple body tags
 * (&lt;body&gt;&lt;/body&gt;).
 *
 * <p>After using this method, the response should be considered
 * to be committed and should not be written to.
 *
 * @param sc the status code
 * @param msg the detail message
 * @exception IOException If an I/O error has occurred.  
 */
public void sendError(int intStatus, String strMsg) throws IOException {
	StringBuffer strb = new StringBuffer();
	strb.append("<html>\n");
	strb.append(" <head>\n");
	strb.append("  <title>Error " + intStatus + "</title>\n");
	strb.append(" </head>\n");
	strb.append(" <body>\n");
	strb.append(strMsg + "\n");
	strb.append("<hr>\n");
	strb.append(Runner.getVersionInfo() + "<br>\n");
	strb.append(" </body>\n");
	strb.append("</html>\n");
	this.intStatus = intStatus;
	this.strStatus = strb.toString();
}
/**
 * Sends a temporary redirect response to the client using the
 * specified redirect location URL.  The URL must be absolute (for
 * example, <code><em>https://hostname/path/file.html</em></code>).
 * Relative URLs are not permitted here.
 *
 * @param location the redirect location URL
 * @exception IOException If an I/O error has occurred.
 */
public void sendRedirect(String strURL) throws IOException {
	strURL = encodeRedirectURL(strURL);
	setHeader("Location", strURL);
	StringBuffer strb = new StringBuffer();
	strb.append("<html>\n");
	strb.append("<head><title>Redirection in progress</title></head>\n");
	strb.append("<body>\n");
	strb.append("The resource you requested has been moved temporarily.<br>\n");
	strb.append("Your browser should automatically take you to the new location.<br>\n");
	strb.append("If not follow the link below to vist the page<br>\n");
	strb.append("<a href=\"" + strURL + "\">" + strURL + "</a><br>\n");
	strb.append("<hr>\n");
	strb.append(Runner.getVersionInfo() + "<br>\n");
	strb.append("</body>\n");
	strb.append("</html>\n");
	this.intStatus = HttpServletResponse.SC_MOVED_TEMPORARILY;
	this.strStatus = strb.toString();
}
/**
 * Adds a field to the response header with the given name and
 * date-valued field.  The date is specified in terms of
 * milliseconds since the epoch.  If the date field had already
 * been set, the new value overwrites the previous one.  The
 * <code>containsHeader</code> method can be used to test for the
 * presence of a header before setting its value.
 * 
 * @param name the name of the header field
 * @param value the header field's date value
 * 
 * @see #containsHeader
 */
public void setDateHeader(String strKey, long lngDate) {
	setHeader(strKey, DateFormat.getDateInstance().format(new Date(lngDate)));
}
/**
 * Adds a field to the response header with the given name and value.
 * If the field had already been set, the new value overwrites the
 * previous one.  The <code>containsHeader</code> method can be
 * used to test for the presence of a header before setting its
 * value.
 * 
 * @param name the name of the header field
 * @param value the header field's value
 *
 * @see #containsHeader
 */
public void setHeader(String strKey, String strData) {
	if (strKey == null || strKey.equals("")) {
		throw new IllegalArgumentException("Wrong parameter: " + strKey);
	}
	dvHeaders.insertOrReplaceDataForKey(strKey, strData);
}
/**
 * Adds a field to the response header with the given name and
 * integer value.  If the field had already been set, the new value
 * overwrites the previous one.  The <code>containsHeader</code>
 * method can be used to test for the presence of a header before
 * setting its value.
 *
 * @param name the name of the header field
 * @param value the header field's integer value
 *
 * @see #containsHeader
 */
public void setIntHeader(String strKey, int intData) {
	setHeader(strKey, Integer.toString(intData));
}
/**
 * Sets the status code for this response.  This method is used to
 * set the return status code when there is no error (for example,
 * for the status codes SC_OK or SC_MOVED_TEMPORARILY).  If there
 * is an error, the <code>sendError</code> method should be used
 * instead.
 *
 * @param sc the status code
 *
 * @see #sendError
 */
public void setStatus(int intStatus) {
	setStatus(intStatus, null);
}
/**
 * Sets the status code and message for this response.  If the
 * field had already been set, the new value overwrites the
 * previous one.  The message is sent as the body of an HTML
 * page, which is returned to the user to describe the problem.
 * The page is sent with a default HTML header; the message
 * is enclosed in simple body tags (&lt;body&gt;&lt;/body&gt;).
 * 
 * @param sc the status code
 * @param sm the status message
 * @deprecated ambiguous meaning. To send an error with a description
 * page, use sendError(int sc, String msg);
 */
public void setStatus(int intStatus, String strStatus) {
	this.intStatus = intStatus;
	this.strStatus = strStatus;
}
/**
 * returns the data for sending to the webserver
 */
public ByteString _getResponseData() {
	ByteString bsInp = super._getResponseData();
	ByteString bsOut = new ByteString(bsInp.getLength() + 1024);

	// check content-type
	if (!dvHeaders.containsKey("Content-Type"))
		dvHeaders.addKeyData("Content-Type", "text/html");

	// Assemble any headers into a single string (buffer)
	for (int i = 0; i < dvHeaders.size(); i++) {
		bsOut.append((String) dvHeaders.keyAt(i) + ": " + (String) dvHeaders.dataAt(i) + "\r\n");
	}
	// cookies
	// see http://www.netscape.com/newsref/std/cookie_spec.html
	SimpleDateFormat dfCookie = new SimpleDateFormat("EEEE, dd-MMM-yyyy HH:mm:ss 'GMT'", java.util.Locale.ENGLISH);
	for (int i = 0; i < vecCookies.size(); i++) {
		Cookie cookie = (Cookie) vecCookies.elementAt(i);
		bsOut.append("Set-Cookie: " + cookie.getName() + "=" + cookie.getValue());
		if (cookie.getVersion() > 0)
			bsOut.append("; version=" + Integer.toString(cookie.getVersion()));
		if (cookie.getPath() != null)
			bsOut.append("; path=" + cookie.getPath());
		else
			bsOut.append("; path=/"); // according to netscape docs
		if (cookie.getComment() != null)
			bsOut.append("; comment=" + cookie.getComment());
		if (cookie.getDomain() != null)
			bsOut.append("; domain=" + cookie.getDomain());
		if (cookie.getMaxAge() > 0) {
			String strTmp = dfCookie.format(new Date(TimeFactory.getTime() + (long) cookie.getMaxAge() * 1000l));
			bsOut.append("; expires=" + strTmp);
		}
		if (cookie.getSecure())
			bsOut.append("; secure");
		bsOut.append("\r\n");
	}

	// we have a statusmessage ? ignore useroutput and bail out
	if (strStatus != null) {
		bsOut.insert(0, "HTTP/1.0 " + intStatus + " SeeCode\r\n");
		bsOut.append("\r\n"); // separate message from header
		bsOut.append(strStatus);
		return bsOut;
	}

	// statuscode only ?
	if (intStatus != 0 && intStatus != HttpServletResponse.SC_OK) {
		bsOut.insert(0, "HTTP/1.0 " + intStatus + " SeeCode\r\n");
	}

	// Separate headers from body if sb. wrote sth before
	if (bsOut.getLength() != 0)
		bsOut.append("\r\n");

	/**
	 * ok, you find the secret. uncomment the next 2 lines and and remove line 3&4 to get rid of the announcement.
	 */
	// the servlet may have no output data written so we return at least a 'blank'
	//if (bsInp.getLength() == 0)
	//	bsInp.append(" ");
	String s = (String) dvHeaders.dataForKey("Content-Type");
	if (s.startsWith("text/"))
		bsInp.insert(0, "<!" + Runner.getVersionInfo() + ">\r\n");
	//
	bsOut.append(bsInp);
	//
	return bsOut;
}
/**
 * 
 */
public void _initObject(HttpServletRequest_Impl srvlRequest) {
	super._initObject();
	this.srvlRequest = srvlRequest;
	vecCookies.removeAllElements();
	bsResponse.clear();
}
}