package net.conacom.servletrunner.impl;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.util.*;
import javax.servlet.*;
import net.conacom.servletrunner.util.*;
import net.conacom.servletrunner.main.*;
/**
 * 
 * Defines a servlet configuration object, which a servlet engine
 * uses to pass information
 * to a servlet in order to initialize the servlet. 
 *
 * <p>The configuration information contains initialization parameters,
 * which are a set of name/value pairs, and a {@link ServletContext} object,
 * which gives the servlet information about the server.
 *
 * @author 	Various
 * @version 	$Version$
 *
 * @see 	ServletContext
 *
 */

/**
 * holds servlet configuration
 *
 * 11.05.02 ho: getInitParamaeter returns NULL if parameter not found
 * 20.03.02 ho: creation
 */
public class ServletConfig_Impl extends Application implements ServletConfig {
	private Properties prpConfig = null;
	private ServletContext srvContext = null;
/**
 * the properties are read by the servletcontainer during initialisation
 */
public ServletConfig_Impl(ServletContext sc, Properties prp) {
	super();
	this.prpConfig = new Properties(prp);
	this.srvContext = sc;
}
/**
 * 
 */
public void dispose() {
	blnDisposed = true;
	prpConfig = null;
	srvContext = null;
}
/**
 * Returns a <code>String</code> containing the value of the 
 * named initialization parameter, or <code>null</code> if 
 * the parameter does not exist.
 *
 * <p>The value of an initialization parameter is a single
 * <code>String</code>, which you must interpret.
 *
 *
 * @param name	a <code>String</code> containing the name
 *			of the parameter whose value is requested
 *
 * @return		a <code>String</code> representing the value 
 *			of the parameter
 *
 */
public String getInitParameter(String strKey) {
	return prpConfig.getProperty(strKey);
}
/**
 * Returns the names of the servlet's initialization parameters
 * as an <code>Enumeration</code> of <code>String</code> objects, 
 * or an empty <code>Enumeration</code> if the servlet has
 * no initialization parameters.
 *
 * @return	an <code>Enumeration</code> of <code>String</code> 
 *			objects containing the names of the servlet's 
 *			initialization parameters
 */
public Enumeration getInitParameterNames() {
	return prpConfig == null ? null : prpConfig.propertyNames();
}
/**
 * Returns the {@link ServletContext} object that the server
 * has passed to this servlet. The <code>ServletContext</code> 
 * object is part of the <code>ServletConfig</code> object this 
 * interface defines.
 *
 *
 * @return		a {@link ServletContext} object, which
 *			gives the servlet information about how
 *			to interact with the server
 * 
 * @see		ServletContext
 */
public ServletContext getServletContext() {
	return srvContext;
}
/**
 * Returns the name of this servlet instance. The name may be provided via server administration, 
 * assigned in the web application deployment descriptor, or for an unregistered (and thus unnamed) 
 * servlet instance it will be the servlet's class name.
 *
 * @return String the name of the servlet instance
 */
public String getServletName() {
	// because all instances are named like "index [1/2]" we can use the thread name
	return Thread.currentThread().getName();
}
}