package net.conacom.servletrunner.impl;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import net.conacom.servletrunner.main.*;
import net.conacom.servletrunner.util.*;
/**
 * Defines a set of methods that a servlet uses to communicate
 * with a servlet engine, for example, to get the MIME type of a file, 
 * locate other servlets running on the server, or
 * write to a servlet log file.
 *
 * <p>The servlet engine talks to the servlet by returning
 * a <code>ServletContext</code> object (defined by this interface)
 * that gives servlets information about their environment. Servlets use the
 * {@link ServletConfig#getServletContext} method to get
 * the <code>ServletContext</code> object.
 *
 * <p>If the server supports
 * multiple or virtual hosts, the <code>ServletContext</code> object
 * must be at least as unique as the host. Servlet engines can also 
 * create <code>ServletContext</code> objects
 * that are unique to a group of servlets
 * and are tied to a specific part of the host's URL namespace.
 * You can assign this grouping administratively or define it in
 * a deployment descriptor file.
 *
 * <p>The <code>ServletContext</code> object is contained within 
 * the {@link ServletConfig} object, which the
 * Web server provides the
 * servlet when the servlet is initialized. You can access
 * the <code>ServletConfig</code> object by using the 
 * {@link Servlet#getServletConfig} method.
 *
 * @author 	Various
 * @version 	$Version$
 *
 * @see 	Servlet#getServletConfig
 * @see 	ServletConfig#getServletContext
 *
 */
/**
 * not implemented:
 * - Set getResourcePaths(String)
 *
 * 27.06.02 ho: getRequestDispatcher(), getNamedDispatcher()
 * 25.06.02 ho: API 2.3
 * 24.06.02 ee: API 2.2
 * 23.05.02 ho: getResource(..), getResourceAsStream(..) is adding the webdir as temp-classpath
 * 21.05.02 ho: getRealPath(String)
 * 14.05.02 ho: default mime-type for unknown files is "application/octet-stream" now
 * 20.03.02 ho: creation
 */
public class ServletContext_Impl extends Application implements ServletContext {
	private ServletContainerConfig scc = null;
	private HttpServletRequest_Impl req = null;
	private ServletConfig srvlConfig = null;
	private ServletInstance si = null;
/**
 * ServletContext_Impl - Konstruktorkommentar.
 */
public ServletContext_Impl(ServletContainerConfig scc, ServletInstance si) {
	super();
	this.scc = scc;
	this.si = si;
}
/**
 * dispose - Methodenkommentar.
 */
public void dispose() {
	blnDisposed = true;
	scc = null;
}
/**
 * Returns the servlet engine attribute with the given name, 
 * or <code>null</code>
 * if there is none. An attribute allows a servlet engine to give the
 * servlet additional information not
 * already provided by this interface. See your
 * Web server documentation for information about its attributes.
 *
 * <p>The attribute is returned as a <code>java.lang.Object</code>.
 * Attribute names should follow the same convention as package
 * names. The Java Servlet API specification reserves names
 * matching <code>java.*</code>, <code>javax.*</code>,
 * and <code>sun.*</code>.
 *
 * @param name 	a <code>String</code> specifying the name 
 *			of the attribute
 *
 * @return 	an <code>Object</code containing the value 
 *			of the attribute, or <code>null</code>
 *			if no attribute exists matching the given name
 */
public Object getAttribute(String strKey) {
	return scc.hshAttributes.get(strKey);
}
/**
 * Returns an <code>Enumeration</code> containing the 
 * attribute names available
 * within this servlet context. You can use the
 * {@link #getAttribute} method with an attribute name
 * to get the value of an attribute.
 *
 * @return 		an <code>Enumeration</code> of attribute 
 *			names
 *
 * @see		#getAttribute
 */
public Enumeration getAttributeNames() {
	return scc.hshAttributes.keys();
}
/**
 * Returns a <code>ServletContext</code> object that 
 * corresponds to a specified URL on the server.
 *
 * <p>This method allows servlets to gain
 * access to the resources located at a specified URL and obtain
 * {@link RequestDispatcher} objects from it.
 * 
 * <p>In security conscious environments, the servlet engine can
 * return <code>null</code> for a given URL.
 *       
 * @param uripath 	a <code>String</code> specifying the URL for 
 *			which you are requesting a <code>ServletContext</code>
 *			object
 *
 * @return		the <code>ServletContext</code> object that
 *			corresponds to the named URL
 *
 * @see 		RequestDispatcher
 *
 */
/**
 * ho: we dont support it for security reasons. period.
 */
public ServletContext getContext(String strURIpath) {
	return null;
}
/**
 * Returns a String containing the value of the named context-wide initialization parameter, 
 * or null if the parameter does not exist.
 *
 * This method can make available configuration information useful to an entire "web application". 
 * For example, it can provide a webmaster's email address or the name of a system that holds critical data.
 *
 * Parameters:
 *   name - a String containing the name of the parameter whose value is requested
 * Returns:
 *   a String containing at least the servlet container name and version number 
 */
public String getInitParameter(String strParamName) {
	return srvlConfig.getInitParameter(strParamName);
}
/**
 * Returns the names of the context's initialization parameters as an Enumeration of String objects, 
 * or an empty Enumeration if the context has no initialization parameters.
 *
 * Returns:
 *   an Enumeration of String objects containing the names of the context's initialization parameters 
 */
public Enumeration getInitParameterNames() {
	return srvlConfig.getInitParameterNames();
}
/**
 * Returns the major version of the Java Servlet API that this
 * Web server supports. All implementations that comply
 * with Version 2.1 must have this method
 * return the integer 2.
 *
 * @return 		2
 *
 */
public int getMajorVersion() {
	return 2;
}
/**
 * Returns the MIME type of the specified file, or <code>null</code> if 
 * the MIME type is not known. The MIME type is determined
 * by the configuration of the servlet engine. Common MIME
 * types are <code>"text/html"</code> and <code>"image/gif"</code>.
 *
 *
 * @param 		a <code>String</code> specifying the name
 *			of the file whose MIME type you want
 *			to check 
 *
 * @return 		a <code>String</code> specifying the MIME type
 *
 */
/**
 * ho: this list is from my OS/2 netscape 4.61 and the Apache JServ
 * 28.04.02 ho: changed behaviour. returns "text/plain" for unknown MIME type
 */
public String getMimeType(String strFile) {
	// "xxx.html" -> "html"
	strFile = strFile.toLowerCase();
	int pp = strFile.indexOf(".");
	if (pp != -1)
		strFile = strFile.substring(pp + 1);
	// text
	if (strFile.equals("html"))
		return "text/html";
	if (strFile.equals("htm"))
		return "text/html";
	if (strFile.equals("txt"))
		return "text/plain";
	if (strFile.equals("css"))
		return "text/css";
	if (strFile.equals("sgml"))
		return "text/x-sgml";
	if (strFile.equals("sgm"))
		return "text/x-sgml";
	// images
	if (strFile.equals("gif"))
		return "image/gif";
	if (strFile.equals("jpg"))
		return "image/jpeg";
	if (strFile.equals("jpeg"))
		return "image/jpeg";
	if (strFile.equals("jpe"))
		return "image/jpeg";
	if (strFile.equals("jpe"))
		return "image/jpeg";
	if (strFile.equals("png"))
		return "image/png";
	if (strFile.equals("tif"))
		return "image/tiff";
	if (strFile.equals("tiff"))
		return "image/tiff";
	if (strFile.equals("rgb"))
		return "image/x-rgb";
	if (strFile.equals("xbm"))
		return "image/x-xbitmap";
	if (strFile.equals("xpm"))
		return "image/x-xpixmap";
	// Audio
	if (strFile.equals("au"))
		return "audio/basic";
	if (strFile.equals("snd"))
		return "audio/basic";
	if (strFile.equals("mid"))
		return "audio/mid";
	if (strFile.equals("midi"))
		return "audio/mid";
	if (strFile.equals("rmi"))
		return "audio/mid";
	if (strFile.equals("kar"))
		return "audio/mid";
	if (strFile.equals("mpga"))
		return "audio/mpeg";
	if (strFile.equals("mp2"))
		return "audio/mpeg";
	if (strFile.equals("mp3"))
		return "audio/mpeg";
	if (strFile.equals("wav"))
		return "audio/wav";
	if (strFile.equals("aiff"))
		return "audio/aiff";
	if (strFile.equals("aifc"))
		return "audio/aiff";
	if (strFile.equals("aif"))
		return "audio/x-aiff";
	if (strFile.equals("ra"))
		return "audio/x-realaudio";
	if (strFile.equals("ram"))
		return "audio/x-pn-realaudio";
	if (strFile.equals("rpm"))
		return "audio/x-pn-realaudio-plugin";
	if (strFile.equals("sd2"))
		return "audio/x-sd2";
	// Application
	if (strFile.equals("bin"))
		return "application/octet-stream";
	if (strFile.equals("dms"))
		return "application/octet-stream";
	if (strFile.equals("lha"))
		return "application/octet-stream";
	if (strFile.equals("lzh"))
		return "application/octet-stream";
	if (strFile.equals("exe"))
		return "application/octet-stream";
	if (strFile.equals("class"))
		return "application/octet-stream";
	if (strFile.equals("hqx"))
		return "application/mac-binhex40";
	if (strFile.equals("ps"))
		return "application/postscript";
	if (strFile.equals("ai"))
		return "application/postscript";
	if (strFile.equals("eps"))
		return "application/postscript";
	if (strFile.equals("pdf"))
		return "application/pdf";
	if (strFile.equals("rtf"))
		return "application/rtf";
	if (strFile.equals("doc"))
		return "application/msword";
	if (strFile.equals("ppt"))
		return "application/powerpoint";
	if (strFile.equals("fif"))
		return "application/fractals";
	if (strFile.equals("p7c"))
		return "application/pkcs7-mime";
	// Application/x
	if (strFile.equals("js"))
		return "application/x-javascript";
	if (strFile.equals("z"))
		return "application/x-compress";
	if (strFile.equals("gz"))
		return "application/x-gzip";
	if (strFile.equals("tar"))
		return "application/x-tar";
	if (strFile.equals("tgz"))
		return "application/x-compressed";
	if (strFile.equals("zip"))
		return "application/x-zip-compressed";
	if (strFile.equals("dir"))
		return "application/postscript";
	if (strFile.equals("dcr"))
		return "application/x-director";
	if (strFile.equals("dxr"))
		return "application/x-director";
	if (strFile.equals("dvi"))
		return "application/x-dvi";
	if (strFile.equals("tex"))
		return "application/x-tex";
	if (strFile.equals("latex"))
		return "application/x-latex";
	if (strFile.equals("tcl"))
		return "application/x-tcl";
	if (strFile.equals("cer"))
		return "application/x-x509-ca-cert";
	if (strFile.equals("crt"))
		return "application/x-x509-ca-cert";
	if (strFile.equals("der"))
		return "application/x-x509-ca-cert";
	// Video
	if (strFile.equals("mpg"))
		return "video/mpeg";
	if (strFile.equals("mpe"))
		return "video/mpeg";
	if (strFile.equals("mpeg"))
		return "video/mpeg";
	if (strFile.equals("qt"))
		return "video/quicktime";
	if (strFile.equals("mov"))
		return "video/quicktime";
	if (strFile.equals("avi"))
		return "video/x-msvideo";
	if (strFile.equals("movie"))
		return "video/x-sgi-movie";
	// Chemical
	if (strFile.equals("pdb"))
		return "chemical/x-pdb";
	if (strFile.equals("xyz"))
		return "chemical/x-pdb";
	// X-
	if (strFile.equals("ice"))
		return "x-conference/x-cooltalk";
	if (strFile.equals("wrl"))
		return "x-world/x-vrml";
	if (strFile.equals("vrml"))
		return "x-world/x-vrml";
	// return text/plain for unknown types.
	return "application/octet-stream";
}
/**
 * Returns the minor version of the Servlet API that this
 * Web server supports. All implementations that comply
 * with Version 2.3 must have this method
 * return the integer 3.
 *
 * @return 		3
 */
public int getMinorVersion() {
	return 3;
}
/**
 * Returns a RequestDispatcher object that acts as a wrapper for the named servlet.
 *
 * Servlets (and JSP pages also) may be given names via server administration or via 
 * a web application deployment descriptor. A servlet instance can determine its name 
 * using ServletConfig.getServletName().
 *
 * This method returns null if the ServletContext cannot return a RequestDispatcher for any reason.
 *
 * Parameters:
 *   name - a String specifying the name of a servlet to wrap
 * Returns:
 *   a RequestDispatcher object that acts as a wrapper for the named servlet 
 */
public RequestDispatcher getNamedDispatcher(String strDispatchName) {
	return si.getRequestDispatcher(strDispatchName, true);
}
/**
 * Returns a <code>String</code> containing the real path 
 * that corresponds to a virtual path. A virtual path contains 
 * a servlet name followed by the name of a file the servlet 
 * should act upon, in the form 
 * <code><i>/dir/dir/servlet/file.ext</i></code>.
 * In this form, <i>file.ext</i> is a filename used instead
 * of the path to the file. The servlet locates the file and 
 * translates the file name to the path that locates the file.
 *
 * <p>The real path the servlet returns is in a form
 * appropriate to the computer and operating system on
 * which the servlet engine is running, including the
 * proper path separators. This method returns <code>null</code>
 * if the servlet engine cannot translate the virtual path
 * to a real path for any reason.
 *
 *
 * @param path 	a <code>String</code> specifying a virtual path,
 *			in the form 
 *			<code><i>/dir/dir/servlet/file.ext</i></code>
 *
 *
 * @return 		a <code>String</code> specifying the real path,
 *			with path separators appropriate for the system
 *			on which the servlet engine is running
 */
/**
 * ho: we use the DOCUMENT_ROOT of the request to translate the path
 */
public String getRealPath(String strPath) {
	strPath = Helper.platformPath(strPath);
	String strRoot = Helper.platformPath((String) req.getAttribute("DOCUMENT_ROOT"));
	if (!strRoot.endsWith(File.separator))
		strRoot = strRoot + File.separator;
	if (strPath.startsWith(File.separator))
		strPath = strPath.substring(1);
	return strRoot + strPath;
}
/**
 * 
 * Returns a {@link RequestDispatcher} object that acts
 * as a wrapper for the resource located at the named path.
 * You can use a <code>RequestDispatcher</code> object to forward 
 * a request to the resource or include a resource in a response.
 *
 * <p>The pathname must be in the form <code>/dir/dir/file.ext</code>.
 * This method returns <code>null</code> if the <code>ServletContext</code>
 * cannot return a <code>RequestDispatcher</code>.
 *
 * <p>The servlet engine is responsible for wrapping the resource
 * with a <code>RequestDispatcher</code> object.
 *
 * @param urlpath 	a <code>String</code> specifying the pathname
 *			to the resource
 *
 * @return 		a <code>RequestDispatcher</code> object
 *			that acts as a wrapper for the resource
 *			at the path you specify
 *
 * @see 		RequestDispatcher
 */
public RequestDispatcher getRequestDispatcher(String strURLpath) {
	return si.getRequestDispatcher(strURLpath, false);
}
/**
 * Returns the resource that is mapped to a specified
 * path. The path must be in the form
 * <code>/dir/dir/file.ext</code>.
 *
 * <p>This method allows the Web
 * server to make a resource available to a servlet from
 * any source. Resources 
 * can be located on a local or remote
 * file system, in a database, or on a remote network site.
 *
 * <p>This method can return <code>null</code>
 * if no resource is mapped to the pathname.
 *
 * <p>The servlet engine must implement the URL handlers
 * and <code>URLConnection</code> objects that are necessary
 * to access the resource.
 *
 * <p>This method has a different purpose than
 * <code>java.lang.Class.getResource</code>,
 * which looks up resources based on a class loader. This
 * method does not use class loaders.
 * 
 * @param path 				a <code>String</code> specifying
 *						the path to the resource,
 * 						in the form <code>/dir/dir/file.ext</code>
 *
 * @return 					the resource located at the named path,
 * 						or <code>null</code> if there is no resource
 *						at that path
 *
 * @exception MalformedURLException 	if the pathname is not given in 
 * 						the correct form
 */
public URL getResource(String strPath) throws MalformedURLException {
	return scc.clLoader.getResource(strPath, (String) req.getAttribute("DOCUMENT_ROOT"));
}
/**
 * 
 * @return java.io.InputStream
 * @param strURLPath java.lang.String
 */
/**
 * ho: implementation notice: this isn't a "real" implementation. it tries to load a file
 *     from the system classpath, the current directory and the "local classpath" for this
 *     servlet instance only.
 */
public InputStream getResourceAsStream(String strURLPath) {
	return scc.clLoader.getResourceAsStream(strURLPath, (String) req.getAttribute("DOCUMENT_ROOT"));
}
/**
 * Returns the name and version of the servlet engine on which
 * the servlet is running. 
 *
 * <p>The form of the returned string is <i>servername</i>/<i>versionnumber</i>.
 * For example, the Java Web Server can return the string
 * <code>Java Web Server/1.1.3</code>.
 *
 * <p>You can design the servlet engine to have this method return 
 * other optional information in parentheses after the primary string, 
 * for example,
 * <code>Java Web Server/1.1.3 (JDK 1.1.6; Windows NT 4.0 x86)</code>.
 *
 *
 * @return 		a <code>String</code> containing at least the 
 *			servlet engine name and version number
 */
public String getServerInfo() {
	return Runner.getVersionInfo();
}
/**
 *
 * @deprecated	As of Java Servlet API 2.1, with no replacement.
 *
 * <p>This method was originally defined to retrieve a servlet
 * from a <code>ServletContext</code>. In this version, this method 
 * always returns <code>null</code> and remains only to preserve 
 * binary compatibility. This method will be permanently removed 
 * in a future version of the Java Servlet API.
 */
public Servlet getServlet(String strName) throws ServletException {
	return null;
}
/**
 * Returns the name of this web application corresponding to this ServletContext as specified in 
 * the deployment descriptor for this web application by the display-name element.
 *
 * Returns:
 *    The name of the web application or null if no name has been declared in the deployment descriptor.
 */
public String getServletContextName() {
	return scc.strServletIdentifier;
}
/**
 * @deprecated	As of Java Servlet API 2.1, with no replacement.
 *
 * <p>This method was originally defined to return an 
 * <code>Enumeration</code>
 * of all the servlet names known to this context. In this version,
 * this method always returns an empty <code>Enumeration</code> and 
 * remains only to preserve binary compatibility. This method will 
 * be permanently removed in a future version of the Java Servlet API.
 */
public Enumeration getServletNames() {
	return Helper.emptyEnumeration();
}
/**
 * <p>This method was originally defined to return an <code>Enumeration</code>
 * of all the servlets known to this servlet context. In this
 * version, this method always returns an empty enumeration and
 * remains only to preserve binary compatibility. This method
 * will be permanently removed in a future version of the Java
 * Servlet API.
 *
 * @deprecated	As of Java Servlet API 2.0, with no replacement.
 */
public Enumeration getServlets() {
	return Helper.emptyEnumeration();
}
/**
 * @deprecated	As of Java Servlet API 2.1, use
 * 			{@link log(String message, Throwable throwable)} 
 *			instead.
 *
 * <p>This method was originally defined to write an 
 * exception's stack trace and an explanatory error message
 * to the servlet log file.
 */
public void log(Exception exception, String msg) {
	throw Helper.deprecatedException();
}
/**
 *
 * Writes the specified message to a servlet log file, which is usually
 * an event log. The message provides explanatory information about
 * an exception or error or an action the servlet engine takes. The name 
 * and type of the servlet log file is specific to the servlet engine.
 *
 *
 * @param msg 	a <code>String</code> specifying the explanatory
 *			message to be written to the log file
 *
 */
public void log(String strMsg) {
	scc.logContext.write("S " + strMsg);
}
/**
 * Writes the stack trace and an explanatory message
 * for a given <code>Throwable</code> exception
 * to the servlet log file. The stack trace is
 * part of the <code>Throwable</code> object, and
 * the message is the one you specify in the <code>message</code>
 * parameter. The name and type of the servlet log file is specific to 
 * the servlet engine, but it is usually an event log.
 *
 *
 * @param message 		a <code>String</code> that 
 *				describes the error or exception
 *
 * @param throwable 	the <code>Throwable</code> error 
 *				or exception
 */
public void log(String strMsg, Throwable thrw) {
	scc.logContext.write("S " + strMsg, thrw);
}
/**
 * Removes the attribute with the given name from 
 * the servlet context. If you remove an attribute, and 
 * then use {@link #getAttribute} to retrieve the 
 * attribute's value, <code>getAttribute</code> returns <code>null</code>.
 *
 *
 * @param name	a <code>String</code> specifying the name 
 * 			of the attribute to be removed
 */
public void removeAttribute(String strName) {
	/**
	 * we don't support removal of conacom servlet runner attributes
	 */
	if (strName.startsWith(Runner.ATTR_PRAEFIX)) {
		throw new RunnerException("Tried to remove a system attribute (" + strName + ")");
	}
	scc.hshAttributes.remove(strName);
}
/**
 *
 * Gives an attribute a name in this servlet context. If
 * the name specified is already used for an attribute, this
 * method will overwrite the old attribute and bind the name
 * to the new attribute.
 *
 * <p>Attribute names should follow the same convention as package
 * names. The Java Servlet API specification reserves names
 * matching <code>java.*</code>, <code>javax.*</code>, and
 * <code>sun.*</code>.
 *
 *
 * @param name 	a <code>String</code> specifying the name 
 *			of the attribute
 *
 *
 * @param object 	an <code>Object</code> representing the
 *			attribute to be given the name
 */
public void setAttribute(String strKey, Object objData) {
	/**
	 * we don't support overwriting of conacom servlet runner attributes
	 */
	if (strKey.startsWith(Runner.ATTR_PRAEFIX)) {
		throw new RunnerException("Tried to overwrite a system attribute (" + strKey + ")");
	}
	scc.hshAttributes.get(strKey);
}
/**
 * initializes some additional parameters depending on the request
 */
public void _initObject(HttpServletRequest_Impl req, ServletConfig srvlConfig) {
	this.req = req;
	this.srvlConfig = srvlConfig;
}
}