package net.conacom.servletrunner.impl;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import javax.servlet.*;
import net.conacom.servletrunner.util.*;
/**
 * 
 * Provides an input stream for reading binary data from a client
 * request, including an efficient <code>readLine</code> method
 * for reading data one line at a time. With some protocols, such
 * as HTTP POST and PUT, you use a <code>ServletInputStream</code>
 * object to read data sent from the client.
 *
 * <p>You usually access a <code>ServletInputStream</code> object
 * by using the {@link ServletRequest#getInputStream} method.
 *
 *
 * <p>This is an abstract class that a servlet engine implements.
 * Subclasses of this class
 * must implement the <code>java.io.InputStream.read</code> method.
 *
 *
 * @author 	Various
 * @version 	$Version$
 *
 * @see		ServletRequest 
 *
 */
/**
 * 07.06.02 ho: fixed bug in read()
 * 21.03.02 ho: creation
 */
public class ServletInputStream_Impl extends ServletInputStream {
	private ByteString bs = null;
	private int intMarkedPos = 0;
	private int intCurrentPos = 0;
/**
 * ServletInputStream_Impl - Konstruktorkommentar.
 */
public ServletInputStream_Impl(ByteString bs) {
	super();
	this.bs = bs;
}
public int available() throws IOException {
	return bs.getLength() - intCurrentPos;
}
public void close() throws IOException {
	if (bs != null) {
		bs.dispose();
		bs = null;
	}
}
public synchronized void mark(int intReadLimit) {
	// we dont have any read limit
	intMarkedPos = intCurrentPos;
}
public boolean markSupported() {
	return true;
}
public int read() {
	if (intCurrentPos >= bs.getLength())
		return -1;
	return bs.getByteAt(intCurrentPos++);
}
public void reset() {
	intCurrentPos = intMarkedPos;
}
}