package net.conacom.servletrunner.impl;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import java.util.*;
import javax.servlet.*;
import net.conacom.servletrunner.util.*;
import net.conacom.servletrunner.main.*;
/**
 * Defines an object that a servlet engine uses to give
 * a servlet information about a client request. 
 *
 * <p>A <code>ServletRequest</code> object provides data, including
 * parameter name and values, attributes, and an input stream.
 * Interfaces that extend <code>ServletRequest</code> can provide
 * additional protocol-specific data (for example, HTTP data is
 * provided by {@link javax.servlet.http.HttpServletRequest}.
 * This interface and the interfaces that descend from
 * it provide the servlet's only access to this data.
 * 
 * <p>A servlet request is a Multipurpose Internet Mail Extension (MIME) 
 * body request, and the response is a MIME body response. MIME bodies
 * are either text or binary data. When they are text,including character
 * encodings, use the <code>getReader</code> method. When they are binary data, 
 * use <code>getInputStream</code>. Multipart MIME bodies are treated
 * as binary data.
 * 
 * @author 	Various
 * @version 	$Version$
 *
 * @see 	javax.servlet.http.HttpServletRequest
 */
/**
 * not implemented for API 2.2:
 * - Map getParameterMap()
 *
 * 25.06.02 ho: API 2.3
 * 24.06.02 ee: API 2.2
 * 10.06.02 ho: RDNS
 * 27.05.02 ho: .getServetPort(): 80 if got port 0
 * 25.03.02 ho: store everything except HTTP_ from the header into the attributes
 * 20.03.02 ho: creation
 */
public class ServletRequest_Impl extends Application implements ServletRequest {
	protected ServletContainerConfig scc = null;
	// this is a local clone of the hashtable used in the ServletSontainerConfig and in the ServletContext_Impl
	// we need a copy because the attributes should be resetted between the requests (as far as i understand the specs)
	protected Hashtable hshAttributes = null;
	// headerdata
	protected Hashtable hshHeader = null;
	// the parsed parameterdata
	protected Hashtable hshParams = null;
	// the request of the server as a copy
	protected ServletInputStream sisRequest = null;
	// retrieved writer/stream already ?
	protected boolean blnGotStream = false;
	// currently constant: send/searh mixedcase headers also (USER_AGENT -> User-Agent)
	protected boolean blnMixedHeader = true;
	// for .getRequestDispatcher(String)
	private ServletInstance si = null;
/**
 * ServletRequest_Impl - Konstruktorkommentar.
 */
public ServletRequest_Impl(ServletContainerConfig scc, ServletInstance si) {
	super();
	this.scc = scc;
	this.si = si;
}
/**
 * 
 */
public void dispose() {
	blnDisposed = true;
	// we're using a copy of the master attributes, so we can clear them here
	if (hshAttributes != null) {
		hshAttributes.clear();
		hshAttributes = null;
	}
	//
	sisRequest = null;
	scc = null;
}
/**
 *
 * Returns the value of the named attribute as an <code>Object</code>. 
 * This method allows the servlet engine to give the servlet
 * custom information about a request. This method returns
 * <code>null</code> if no attribute of the given name exists.
 *
 * <p>Attribute names should follow the same conventions as package
 * names. This specification reserves names matching <code>java.*</code>,
 * <code>javax.*</code>, and <code>sun.*</code>. 
 *
 * @param name	a <code>String</code> specifying the name of 
 *			the attribute
 *
 * @return		an <code>Object</code> containing the value 
 *			of the attribute, or <code>null</code> if
 *			the attribute does not exist
 */
public Object getAttribute(String strKey) {
	Object o = hshAttributes.get(strKey);
	if (o == null && blnMixedHeader)
		o = hshAttributes.get(Helper.headerkey2cgikey(strKey));
	return o;
}
/**
 * Returns an <code>Enumeration</code> containing the
 * names of the attributes available to this request. 
 * This method returns an empty <code>Enumeration</code>
 * if the request has no attributes available to it.
 * 
 *
 * @return		an <code>Enumeration</code> of strings 
 *			containing the names 
 * 			of the request's attributes
 */
public Enumeration getAttributeNames() {
	if (!blnMixedHeader)
		return hshAttributes.keys();
	//
	Hashtable hsh = new Hashtable();
	for (Enumeration e = hshAttributes.keys(); e.hasMoreElements();) {
		String s = (String) e.nextElement();
		hsh.put(s, "");
		hsh.put(Helper.cgikey2headerkey(s), "");
	}
	return hsh.keys();
}
/**
 * helpermethod for retrieving a string from the CGI environment
 */
protected final String[] getCGI_Environment(String strKey) {
	if (strKey == null)
		return null;
	Object o[] = (Object[]) hshHeader.get(strKey);
	// "User-Agent" in ? try USER_AGENT
	if (o == null && blnMixedHeader)
		o = (Object[]) hshHeader.get(Helper.headerkey2cgikey(strKey));
	return o == null ? null : (String[]) o;
}
/**
 * helpermethod for retrieving a string from the CGI environment
 */
protected final String getCGI_Environment0(String strKey) {
	String s[] = getCGI_Environment(strKey);
	return s == null ? null : s[0];
}
/**
 * Returns the name of the character encoding style used in this
 * request. This method returns <code>null</code> if the request
 * does not use character encoding. 
 * 
 *
 * @return	a <code>String</code> containing the name of 
 *			the chararacter encoding style, or <code>null</code>
 *			if the request does not use character encoding
 *
 */
public String getCharacterEncoding() {
	int pp = -1;
	String str = getContentType();
	if (str != null)
		pp = str.indexOf("charset=");
	if (pp != -1) {
		// isolate chars after 'charset='
		str = str.substring(pp + 8);
		//
		pp = str.indexOf(";");
		return (pp != -1) ? str.substring(0, pp) : str;
	}
	// not found try ACCEP_CHARSET ? 
	Enumeration e = Helper.listEnumerator(getCGI_Environment0("HTTP_ACCEPT_CHARSET"));
	if (e.hasMoreElements())
		return (String) e.nextElement();
	// return default according to specs
	return "ISO-8859-1";
}
/**
 * Returns the length, in bytes, of the content contained in the
 * request and sent by way of the input stream or -1 if the
 * length is not known. Same as the value
 * of the CGI variable CONTENT_LENGTH.
 *
 * @return		an integer containing the length of the content 
 * 			in the request or -1 if the length is not known
 */
public int getContentLength() {
	String s = getCGI_Environment0("CONTENT_LENGTH");
	return s == null ? -1 : Integer.parseInt(s);
}
/**
 * Returns the MIME type of the content of the request, or 
 * <code>null</code> if the type is not known. Same as the value
 * of the CGI variable CONTENT_TYPE.
 *
 * @return	a <code>String</code> containing the name 
 *			of the MIME type of 
 * 			the request, or -1 if the type is not known
 */
public String getContentType() {
	String s = getCGI_Environment0("CONTENT_TYPE");
	return s == null ? "text/plain" : s;
}
/**
 * Retrieves binary data from the body of the request as 
 * a {@link ServletInputStream}, which
 * gives you the ability to read one line at a time.
 *
 * @return					a {@link ServletInputStream} object containing
 * 						the body of the request
 *
 * @exception IllegalStateException   	if the {@link #getReader} method
 * 						has already been called for this request
 *
 * @exception IOException    		if an input or output exception occurred
 */
public ServletInputStream getInputStream() throws IOException {
	if (blnGotStream)
		throw new IllegalStateException("Inputstream already requested");
	blnGotStream = true;
	return sisRequest;
}
/**
 * Returns the preferred Locale that the client will accept content in, based on the 
 * Accept-Language header. If the client request doesn't provide an Accept-Language header, 
 * this method returns the default locale for the server.
 *
 * Returns:
 *   the preferred Locale for the client
 */
public Locale getLocale() {
	return (Locale) getLocales().nextElement();
}
/**
 * Returns an Enumeration of Locale objects indicating, in decreasing order starting with the preferred 
 * locale, the locales that are acceptable to the client based on the Accept-Language header. If the client 
 * request doesn't provide an Accept-Language header, this method returns an Enumeration containing one Locale, 
 * the default locale for the server.
 *
 * Returns:
 *   an Enumeration of preferred Locale objects for the client 
 */
public Enumeration getLocales() {
	Vector v = new Vector();
	Enumeration e = Helper.listEnumerator(getCGI_Environment0("HTTP_ACCEPT_LANGUAGE"));
	while (e.hasMoreElements()) {
		String s = (String) e.nextElement();
		s = s.replace('-', '_');
		String strLeft = "";
		String strRight = "";
		int pp = s.indexOf("_");
		if (pp != -1) {
			strLeft = s.substring(0, pp);
			strRight = s.substring(pp + 1);
		} else {
			strLeft = s;
			strRight = s;
		}
		v.addElement(new Locale(strLeft.toLowerCase(), strRight.toUpperCase()));
	}
	// empty ? return default locale
	if (v.size() == 0)
		v.addElement(Locale.getDefault());
	return v.elements();
}
/**
 * Returns the value of a request parameter as a <code>String</code>,
 * or <code>null</code> if the parameter does not exist. Request parameters
 * are extra information sent with the request. 
 *
 * <p>You should only use this method when you are sure the
 * parameter has only one value. If the parameter might have
 * more than one value, use {@link #getParameterValues}.
 *
 * <p>If you use this method with a multivalued
 * parameter, the servlet engine determines the return value.
 *
 *
 * @param name 	a <code>String</code> specifying the 
 *			name of the parameter
 *
 * @return		a <code>String</code> representing the 
 *			single value of the parameter
 *
 * @see 		#getParameterValues
 */
public String getParameter(String strKey) {
	try {
		return ((String[]) hshParams.get(strKey))[0];
	} catch (NullPointerException e) {
		return null;
	}
}
/**
 * Returns an <code>Enumeration</code> of <code>String</code>
 * objects containing the names of the parameters contained
 * in this request. If the request has 
 * no parameters or if the input stream is empty, returns an 
 * empty <code>Enumeration</code>. The input stream is empty 
 * when all the data returned by {@link #getInputStream} has 
 * been read.
 *
 * @return	an <code>Enumeration</code> of <code>String</code>
 *			objects, each <code>String</code> containing
 * 			the name of a request parameter; or an 
 *			empty <code>Enumeration</code> if the
 *			request has no parameters
 */
public Enumeration getParameterNames() {
	return hshParams.keys();
}
/**
 * Returns an array of <code>String</code> objects containing 
 * all of the values the
 * given request parameter has, or <code>null</code> if the 
 * parameter does not exist. For example, in an HTTP servlet, 
 * this method returns an array of <code>String</code> objects 
 * containing the values of a query string or posted form.
 *
 * <p>If the parameter has a single value, the array has a length
 * of 1.
 *
 * @param name	a <code>String</code> containing the name of 
 *			the parameter whose value is requested
 *
 * @return		an array of <code>String</code> objects 
 *			containing the parameter's values
 *
 * @see		#getParameter
 */
public String[] getParameterValues(String name) {
	return (String[]) hshParams.get(name);
}
/**
 * Returns the name and version of the protocol the request uses
 * in the form <i>protocol/majorVersion.minorVersion</i>, for 
 * example, HTTP/1.1. The value
 * returned is the same as the value of the CGI variable 
 * <code>SERVER_PROTOCOL</code>.
 *
 * @return		a <code>String</code> containing the protocol 
 *			name and version number
 */
public String getProtocol() {
	String s = getCGI_Environment0("SERVER_PROTOCOL");
	return s == null ? "HTTP/1.0" : s;
}
/**
 * Returns the body of the request as a <code>BufferedReader</code>
 * that translates character set encodings.
 * 
 *
 * @return					a <code>BufferedReader</code>
 *						containing the body of the request	
 *
 * @exception UnsupportedEncodingException 	if the character set encoding
 * 						used is not supported and the 
 *						text cannot be decoded
 *
 * @exception IllegalStateException   	if {@link #getInputStream} method
 * 						has been called on this request
 *
 * @exception IOException  			if an input or output exception occurred
 *
 * @see 					#getInputStream
 */
public BufferedReader getReader() throws IOException {
	if (blnGotStream)
		throw new IllegalStateException("Inputstream already requested");
	blnGotStream = true;
	return new BufferedReader(new InputStreamReader(sisRequest));
}
/**
 * @deprecated 	As of Version 2.1 of the Java Servlet API,
 * 			use {@link ServletContext#getRealPath} instead.
 */
public String getRealPath(String path) {
	throw Helper.deprecatedException();
}
/**
 * Returns the Internet Protocol (IP) address of the client 
 * that sent the request.
 * Same as the value of the CGI variable <code>REMOTE_ADDR</code>.
 *
 * @return	a <code>String</code> containing the 
 *			IP address of the client that sent the request
 */
public String getRemoteAddr() {
	return getCGI_Environment0("REMOTE_ADDR");
}
/**
 * Returns the fully qualified name of the client that sent the
 * request. Same as the value of the CGI variable <code>REMOTE_HOST</code>.
 *
 * @return		a <code>String</code> containing the fully qualified name 
 *			of the client
 */
public String getRemoteHost() {
	String s = getCGI_Environment0("REMOTE_HOST");
	if (s == null || s.length() == 0)
		return getCGI_Environment0("REMOTE_ADDR");
	// RDNS ?
	if (scc.blnRDNS) {
		try {
			java.net.InetAddress ina = java.net.InetAddress.getByName(s);
			s = ina.getHostName();
		} catch (Throwable t) {
		}
	}
	return s;
}
/**
 * Returns a RequestDispatcher object that acts as a wrapper for the resource located at the 
 * given path. A RequestDispatcher object can be used to forward a request to the resource or 
 * to include the resource in a response. The resource can be dynamic or static.
 *
 * The pathname specified may be relative, although it cannot extend outside the current servlet 
 * context. If the path begins with a "/" it is interpreted as relative to the current context root. 
 * This method returns null if the servlet container cannot return a RequestDispatcher.
 *
 * The difference between this method and ServletContext.getRequestDispatcher(java.lang.String) is that 
 * this method can take a relative path.
 *
 * Parameters:
 *   path - a String specifying the pathname to the resource
 * Returns:
 *   a RequestDispatcher object that acts as a wrapper for the resource at the specified path
 */
public RequestDispatcher getRequestDispatcher(String strDispatchPath) {
	return si.getRequestDispatcher(strDispatchPath, false);
}
/**
 * Returns the name of the scheme used to make this request, 
 * for example,
 * <code>http</code>, <code>https</code>, or <code>ftp</code>.
 * Different schemes have different rules for constructing URLs,
 * as noted in RFC 1738.
 *
 * <p>You can reconstruct the URL used to make this request by 
 * using this scheme, the server name and port, the pathname to
 * the Web page on the server (also known as the Universal 
 * Resource Identifier), and the query string..
 *
 * @return	a <code>String</code> containing the name 
 *			of the scheme used to make this request
 */
public String getScheme() {
	String strURL = getCGI_Environment0("SERVER_URI");
	if (strURL != null) {
		int pp = strURL.indexOf(":");
		if (pp != -1)
			return strURL.substring(0, pp);
	}
	// Default scheme, if we cannot determine it
	return "http";
}
/**
 * Returns the host name of the server that received the request.
 * Same as the value of the CGI variable <code>SERVER_NAME</code>.
 *
 * @return		a <code>String</code> containing the name 
 *			of the server to which the request was sent
 */
public String getServerName() {
	String s = getCGI_Environment0("SERVER_NAME");
	if (s == null || s.length() == 0)
		return s;
	// RDNS ?
	if (scc.blnRDNS) {
		try {
			java.net.InetAddress ina = java.net.InetAddress.getByName(s);
			s = ina.getHostName();
		} catch (Throwable t) {
		}
	}
	return s;
}
/**
 * Returns the port number on which this request was received.
 * Same as the value of the CGI variable <code>SERVER_PORT</code>.
 *
 * @return		an integer specifying the port number
 *
 */
/**
 * 27.05.02 ho: HttpUtils#getRequestURL() appends the port, but Xitami sends 0
 */
public int getServerPort() {
	int intPort = 0;
	String s = getCGI_Environment0("SERVER_PORT");
	if (s != null)
		intPort = Integer.parseInt(s);
	return intPort == 0 ? 80 : intPort;
}
/**
 * Returns a boolean indicating whether this request was made using a secure channel, such as HTTPS.
 *
 * Returns:
 *  a boolean indicating if the request was made using a secure channel
 */
public boolean isSecure() {
	return getProtocol().toUpperCase().startsWith("HTTPS");
}
/**
 * Removes an attribute from this request. This method is not generally needed as attributes 
 * only persist as long as the request is being handled.
 *
 * Attribute names should follow the same conventions as package names. Names beginning with 
 * java.*, javax.*, and com.sun.*, are reserved for use by Sun Microsystems.
 *
 * Parameters:
 *   name - a String specifying the name of the attribute to remove
 */
public void removeAttribute(String strKey) {
	if (strKey == null || strKey.equals(""))
		return;
	hshAttributes.remove(strKey);
}
/**
 * Stores an attribute in the context of this request.
 * Attributes are reset between requests.
 *
 * <p>Attribute names should follow the same conventions as
 * package names. Names beginning with <code>java.*</code>,
 * <code>javax.*</code>, and <code>com.sun.*</code>, are
 * reserved for use by Sun Microsystems.
 *
 *
 * @param key			a <code>String</code> specifying 
 *					the name of the attribute
 *
 * @param o				an <code>Object</code> containing 
 *					the context of the request
 *
 * @exception IllegalStateException	if the specified attribute already has a value
 *
 */
public void setAttribute(String strKey, Object objData) {
	hshAttributes.put(strKey, objData);
}
/**
 * Overrides the name of the character encoding used in the body of this request. This method must be called 
 * prior to reading request parameters or reading input using getReader().
 *
 * Parameters:
 *   a String containing the name of the chararacter encoding.
 */
/**
 * does nothing
 */
public void setCharacterEncoding(String strEncoding) throws UnsupportedEncodingException {

}
/**
 * HELPER: returns the header hashtable (eg. for RequestDispatcher)
 */
public Hashtable _getHeader() {
	return hshHeader;
}
/**
 * HELPER: returns the current input stream (eg. for RequestDispatcher)
 */
public ServletInputStream _getInputStream() {
	return sisRequest;
}
/**
 * HELPER: returns the param hashtable (eg. for RequestDispatcher)
 */
public Hashtable _getParams() {
	return hshParams;
}
/**
 * initializes this object.
 */
public void _initObject(Hashtable hshHeader, Hashtable hshParams, ServletInputStream sisRequest) {
	this.hshAttributes = (Hashtable) scc.hshAttributes.clone();
	this.hshHeader = hshHeader;
	this.hshParams = hshParams;
	this.sisRequest = sisRequest;
	// copy all request-parameters except HTTP_ into the attributes
	for (Enumeration e = hshHeader.keys(); e.hasMoreElements();) {
		String s = (String) e.nextElement();
		if (!s.startsWith("HTTP_")) {
			String strData[] = (String[]) hshHeader.get(s);
			hshAttributes.put(s, strData[0]);
		}
	}
	// init
	blnGotStream = false;
}
}