package net.conacom.servletrunner.main;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 *
 * Portions copyright iMatix New Zealand (http://www.imatix.co.nz), iMatix Belgium (http://www.imatix.com)
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import java.net.*;
import java.util.*;
import net.conacom.servletrunner.util.*;
import net.conacom.servletrunner.sessionmanager.*;
/**
 * The runner itself. Spawns Servlet containers.
 *
 * Author notice: 
 * 'ho:' is bernd hohmann (hohmann@conacom.de)
 * 'ks:' is karl schmidt (schmidt@conacom.de)
 * 'ee:' is ender erbey (ender.erbey@conacom.de)
 *
 * - [to do] ------------------------
 * request dispatcher
 * restart or shutdown a container instead of all servlets
 * ----------------------------------
 * 17.07.02 ho: **1.7
 * 				Watchdog is started in the middle of .init() now.
 *				Fixed some bugs in restarting the runner
 * 01.07.02 ho: **1.6
 *				ServletContext#getRequestDispatcher(..), #getNamedDispatcher(..)
 *				Information about operating system logged
 *				Attribute javax.servlet.context.tempdir points now to a java.io.File with a tempdir
 *				Runner#check_semaphore() is a thread now.
 * 				(ho) update to API 2.3, not implemented are methods with Java 1.2 return values (Map, Set)
 *				(EE) update to API 2.2
 *				Session-ID is configurable in the sessionmanager config now (session_id)
 *				SessionCookie expires with -1 (browser session) again.
 *				removed "queued_startup" parameter. ServletInstance uses LockObject for synchronizing .init() 
 *				and .run() which allows faster startup.
 * 16.06.02 ho: **1.5
 *				Speedups in MonitoredExecution (caching, thread reusing). 10 times faster now (hard work)
 *				Event system for shutdown/dispose
 *				Fixed problem with various un-closed streams (Properties)
 * 14.06.02 ho: **1.4
 *				RunnerClassLoader: added local path to container classpath if classpath is empty
 *				getServetName() & getRemoteHost() returning RDNS name
 *				reverse_dns=1/0 in container config
 *				added logscn_options
 *              ServletInstance#procesError(..): uses rootcause of error now
 * 07.06.02 ho: **1.3
 *				HttpServletResponse: don't encode redirect URL if
 *				URL starts with a protocol (http://, ftp://...)
 *				LogContext getDateFormat(),getTimeFormat().
 *				RemoteManager.
 * 01.06.02 ho:	**1.2
 *				HttpServletResponse: fix in redirectURL
 *				ServletRequest#getServetPort(): 80 if got port 0
 * 24.05.02 ho:	**1.1
 *				RunnerClassLoader: webdir is added to the classpath (ServetContext)
 *				RunnerClassLoader: closing open zipfiles on .dispose();
 *				RunnerClassLoader: fixed getResource(..), getResourceAsStream(..)
 * 				RunnerClassLoader: getResource(..) and getResourceAsStream(..) 
 *				are removing leading / from the file now.
 *				Sessioncookie expiry time according to sessionmanager timeout
 *              LogContext#writeMessage(Throwable)
 *              MonitoredException returns Rootcause for ServletException 
 * 21.05.02 ho: **1.0
 *				ServletContext_Impl#getRealPath(String)
 *				TestServlet: list ServletContext methods
 *				Fixed some glitches in MonitoredExcecution
 * 19.05.02 ho: **0.9-rc-10
 *				Fixed getRequestURI()
 *				HttpServletRequest_Impl will return attributes for both "USER_AGENT" and "User-Agent" and other keys
 * 16.05.02 ho: **0.9-rc-9
 *				ServletContext#getMimeType(): default mime-type for unknown files is "application/octet-stream" now.
 *				Runner: the first commandline parameter is accepted as configurationfile (runner.ini)
 *				ServletInstance: creation of the instance, calling .init(...), .service(...) and .destroy() are monitored.
 *				If the action doesn't finishes after a given time, thecall will be interrupted (monitor_* parameters in the
 *              servlet configuration).
 * 14.05.02 ho: **0.9-rc-8
 *				Experimental Support for "URL getResource()".
 *              Fixed Problem setting date/timeformat for logging
 * 13.05.02 ho: **0.9-rc-7
 *				config.variable=value in servlet.ini
 *				Some HttpServletRequest methods are returning a platform dependent path now
 * 11.05.02 ho: **0.9-rc-6
 *				loadFromStream(..) gots file-formatted string instead of the original classname, which causes troubles on some JVMs
 *				FileSessionmanager: fixed a bug in an errormessage
 *				ServletConfig_Impl returns NULL on unknown parameters now (according to the specs)
 *				Container-Messages during startup are in the logContext of the container now
 *				if an extra logfile is used, the context-root is prepended to the logfile_path
 * 07.05.02 ho: **v0.9-rc-5
 *				merged LogWriter and LogContext, prepared everything for splitting the log-filter for screen and file-
 *				fixed bug in RunnerClassLoader causing NullPointerException when the additional classpath was damaged
 *				RunnerClassLoader now displays an errormessage when subclasses cannot be loaded.
 *				added "logfile_remoe_classname" option.
 * 28.04.02 ho: **v0.9-rc-r4
 *				RunnerClassLoader#getResourceAsStream(...)
 *              ServletContext_Imp#getRespourceAsStream(...)
 *				HttpServletResponse_Impl#_getResponseData() inserts a versionheader into the HTTP response now.
 *				Runner#getVersionInfo()
 *				ATTR_PRAEFIX
 *				Helper.getDirectory(..)
 *				path in srvl_config causes adjustment of logfile_path, classpath, config
 *				fixed bug in redirect again.
 * 19.04.02 ho: **v0.9-rc-r3 retries to connect to the webserver (lrwp_retries, lrwp_retrydelay)
 *              better dispose(). TestServlet: bigdata test for xitami 2.4d9
 * 12.04.02 ho: **v0.9-rc-r2 performance update, stresstest, date/timeformat
 * 09.04.02 ho: **v0.9-rc bugfixing, better testservlet
 * 03.04.02 ho: **v0.8 support for external GUI
 * 02.04.02 ho: **v0.7 version
 * 29.03.02 ho: some code-cleanup, more documentations
 *				supports sessions from URL now (containerconfig "sessionid=url/cookie")
 *				new implementations:
 *				ServletRequest_Impl#getCharacterEncoding()
 *				ServletRequest_Impl#getSessionId()
 *				ServletRequest_Impl#getSessionIDFromURL()
 *				ServletRequest_Impl#getSessionIDFromCookie()
 *				ServletResponse_Implo#encodeRedirectURL(..)
 *				ServletResponse_Impl#encodeURL(..)
 *				ServletContext_Impl#getMimeType(...)
 * 28.03.02 ho: ** v0.6 version
 * 27.03.02 ho: garbled output in xitami when no headerfield was set (HttpResponse#_getResponseData()
 * 26.03.02 ho: ** v0.5 NEW: RunnerClassLoader. added copyright, released sourcecode
 * 25.03.02 ho: ** v0.4 fixed: parameters were not parsed. check for formdata/multipart
 * 25.03.02 ho: ** v0.3 minor bugfixes (opening logfilepath, shutdown etc..). NEW: FileSessionManager
 * 23.03.02 ho: ** v0.2 misc bugfixing, processname, nicer logging, raceconditions when startup
 * 22.03.02 ho: ** v0.1 finished 
 * 13.03.02 ho: creation
 */
public class Runner extends Application implements RunnerEventListener, Runnable {
	// the configurationfile to use
	private static String CONFIGFILE = "runner.ini";
	// the version (-> getVersionInfo())
	private final static String VERSION = "1.7";
	// praefix for servlet attributes (SSR=sokrates servlet runner) 
	// see: Runner#createContainer, ServletContext_Impl#removeAttribute, ServletContext_Impl#setAttribute
	// check HttpSessionImpl#SESSION_IDENTIFIER also.
	public final static String ATTR_PRAEFIX = "net.conacom.servletrunner.";
	// logcontext for the runner itself
	private LogContext logContext = null;
	// all containers started by this runner (for .dispose())
	private Vector vecContainers = null;
	// the runner configuration
	private Properties prpRunnerConfig = null;
	// the sessionmanager. its global because we have to dispose him
	private SessionManager smManager = null;
	// remotecontrol
	private static RemoteControlConfig rcc = null;
	// sempaphore check thread
	private Thread thrdSemaCheck = null;
	private LockObject objNotify = null;
	// root of temporary directory
	private File flTempRoot = null;
/**
 * creates a new instance of the servlet-runner
 */
public Runner() {
	super();
	init();
}
/**
 * reads a config for a special servlet and creates a container for it
 */
private void createContainer(String strConfigFile) {
	/**
	 * some defaults for the container
	 */
	Properties prpContainer = new Properties();
	prpContainer.put("servletidentifier", "/servlet");
	prpContainer.put("class", "net.conacom.servletrunner.util.TestServlet");
	prpContainer.put("instances", "1");
	prpContainer.put("virtualhost", "");
	prpContainer.put("host", prpRunnerConfig.getProperty("default_host"));
	prpContainer.put("port", prpRunnerConfig.getProperty("default_port"));
	prpContainer.put("logfile_path", "");
	prpContainer.put("logfile_praefix", "");
	prpContainer.put("logfile_suffix", "");
	prpContainer.put("logfile_options", "");
	prpContainer.put("logscrn_options", "");
	prpContainer.put("logfile_dateformat", "dd.MM.yyyy");
	prpContainer.put("logfile_timeformat", "HH:mm:ss");
	prpContainer.put("logfile_remove_classname", "0");
	prpContainer.put("config", ""); // additional configuration properties
	prpContainer.put("outputbuffer", "10000");
	prpContainer.put("sessiontimeout", "60");
	prpContainer.put("processname", "");
	prpContainer.put("classpath", "");
	prpContainer.put("sessionid", "cookie");
	prpContainer.put("lrwp_retries", "5"); // number of retries connecting to the webserver
	prpContainer.put("lrwp_retrydelay", "2000"); // number of retries connecting to the webserver
	prpContainer.put("monitor_instance", "5000"); // timeout for creating the servletinstance
	prpContainer.put("monitor_init", "15000"); // timeout for calling .init(...)
	prpContainer.put("monitor_service", "30000"); // timeout for calling .service(....)
	prpContainer.put("monitor_destroy", "15000"); // timeout for calling .destroy()
	prpContainer.put("reverse_dns", "1"); // reverse dns on ServletRequest#getRemoteHost() ?

	/**
	 * load servlet-configuration
	 */
	InputStream in = null;
	try {
		in = new FileInputStream(strConfigFile);
		prpContainer.load(in);
	} catch (IOException e) {
		logContext.write("? Error loading servletconfig '" + strConfigFile + "': " + e.toString());
		return;
	} finally {
		Helper.close(in);
	}

	// the context-root is the directory where the configfile of this container resides
	String strContextRoot = Helper.getDirectory(strConfigFile);
	if (strContextRoot == null) {
		logContext.write("? Cannot find the path in " + strConfigFile);
		return;
	}

	/**
	 * copies the value from the merged properties into a new ServletContainerConfig
	 */
	ServletContainerConfig scc = new ServletContainerConfig();
	scc.strConfigFile = strConfigFile;
	// LRWP
	scc.strLRWP_Host = prpContainer.getProperty("host");
	scc.strLRWP_Port = prpContainer.getProperty("port");
	try {
		scc.intLRWP_Retries = Integer.parseInt(prpContainer.getProperty("lrwp_retries"));
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unable to parse your 'lrwp_retries' parameter");
	}
	try {
		scc.lngLRWP_RetryDelay = Integer.parseInt(prpContainer.getProperty("lrwp_retrydelay"));
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unable to parse your 'lrwp_retrydelay' parameter");
	}
	// servlet
	scc.strServletIdentifier = prpContainer.getProperty("servletidentifier");
	scc.strVirtualHost = prpContainer.getProperty("virtualhost");
	scc.strClass = prpContainer.getProperty("class");
	scc.intNbOfInstances = Integer.parseInt(prpContainer.getProperty("instances"));
	// logging
	boolean b = false; // logfile parameters overwrite ? create another logfile
	b |= prpContainer.getProperty("logfile_path").length() != 0;
	b |= prpContainer.getProperty("logfile_praefix").length() != 0;
	b |= prpContainer.getProperty("logfile_suffix").length() != 0;
	b |= prpContainer.getProperty("logfile_options").length() != 0;
	if (b) {
		String strLogPath = strContextRoot + prpContainer.getProperty("logfile_path");
		prpContainer.put("logfile_path", strLogPath);
		scc.logContext = createLogContext(prpContainer);
		scc.blnOwnLogger = true;
		logContext.write("# " + scc.strServletIdentifier + " is using separate logcontext in " + strLogPath);
	} else {
		scc.logContext = logContext;
		scc.blnOwnLogger = false;
		logContext.write("# " + scc.strServletIdentifier + " is using servlet runner logcontext");
	}
	/**
	 * from here, were using scc.logContext instead of logContext
	 */
	// additional container config
	scc.prpConfig = new Properties();
	if (prpContainer.getProperty("config").length() != 0) {
		// append contextroot if possible
		String s = prpContainer.getProperty("config");
		if (Helper.getDirectory(s).length() == 0)
			s = strContextRoot + s;
		try {
			scc.prpConfig.load(new FileInputStream(s));
		} catch (Throwable t) {
			scc.logContext.write("! Cannot load properties from " + prpContainer.getProperty("config") + ": " + t.getMessage());
		}
	}
	// config.blahblah = something ? remove "config." and put it into properties
	for (Enumeration e = prpContainer.keys(); e.hasMoreElements();) {
		String strKey = (String) e.nextElement();
		if (strKey.toLowerCase().startsWith("config.")) {
			scc.prpConfig.put(strKey.substring(7), prpContainer.getProperty(strKey));
		}
	}
	// peanuts
	scc.blnRDNS = prpContainer.getProperty("reverse_dns").equals("1");
	scc.intOutputBufferSize = Integer.parseInt(prpContainer.getProperty("outputbuffer"));
	scc.smManager = smManager;
	if (prpContainer.getProperty("processname").length() == 0)
		scc.strProcessName = scc.strServletIdentifier;
	else
		scc.strProcessName = prpContainer.getProperty("processname");
	scc.clLoader = new RunnerClassLoader(prpContainer.getProperty("classpath"), strContextRoot, scc.logContext);
	scc.blnUseCookieForSession = !prpContainer.getProperty("sessionid").equals("url");
	scc.logContext.write("# Using " + (scc.blnUseCookieForSession ? "Cookies" : "URL") + " for storing the session-id");
	// monitoring
	try {
		scc.lngMonitor_instance = Integer.parseInt(prpContainer.getProperty("monitor_instance"));
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unable to parse your 'monitor_instance' parameter");
	}
	try {
		scc.lngMonitor_init = Integer.parseInt(prpContainer.getProperty("monitor_init"));
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unable to parse your 'monitor_init' parameter");
	}
	try {
		scc.lngMonitor_service = Integer.parseInt(prpContainer.getProperty("monitor_service"));
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unable to parse your 'monitor_service' parameter");
	}
	try {
		scc.lngMonitor_destroy = Integer.parseInt(prpContainer.getProperty("monitor_destroy"));
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unable to parse your 'monitor_destroy' parameter");
	}
	/**
	 * temporary directory
	 */
	File flTmpDir = new File(flTempRoot, scc.strProcessName);
	try {
		flTmpDir.mkdirs();
	} catch (Throwable t) {
		scc.logContext.write("? Cannot create tempdir for container. (" + flTmpDir + "). Aborting", t);
		return;
	}
	scc.logContext.write("# Using tempdir " + flTmpDir);

	/**
	 * set some internal attributes, maybe sb. need them
	 * can be retrieved with getServletContext().getAttribute ... or in the ServletRequest as a copy
	 */
	scc.hshAttributes = new Hashtable();
	scc.hshAttributes.put(Runner.ATTR_PRAEFIX + "usecookies", new Boolean(scc.blnUseCookieForSession));
	scc.hshAttributes.put(Runner.ATTR_PRAEFIX + "sessionmanager", smManager.getClass().getName());
	scc.hshAttributes.put(Runner.ATTR_PRAEFIX + "servletidentifier", scc.strServletIdentifier);
	scc.hshAttributes.put(Runner.ATTR_PRAEFIX + "contextroot", strContextRoot);
	scc.hshAttributes.put("javax.servlet.context.tempdir", flTmpDir);

	/**
	 * find additional ressources for the container (import=)	 
	 */
	scc.hshImports = new Hashtable();
	BufferedReader br = null;
	try {
		br = new BufferedReader(new FileReader(strConfigFile));
		String strBuffer = null;
		while ((strBuffer = br.readLine()) != null) {
			strBuffer = strBuffer.trim();
			int pp = strBuffer.indexOf("=");
			if (pp == -1)
				continue;
			String strKey = strBuffer.substring(0, pp).trim();
			String strRes = strBuffer.substring(pp + 1).trim();
			if (strKey.equalsIgnoreCase("import")) {
				pp = strRes.indexOf(",");
				String strCall, strAlias;
				if (pp != -1) {
					strCall = strRes.substring(0, pp).trim();
					strAlias = strRes.substring(pp + 1).trim();
				} else {
					strCall=strRes;
					strAlias=strRes;
				}
				scc.hshImports.put(strAlias, strCall);
			}
		}
		br.close();
	} catch (IOException e) {
		Helper.close(br);
		logContext.write("? Cannot read container configuration. Container not started", e);
		return;
	}

	/**
	 * have config, will travel !
	 */
	scc.logContext.write("# Starting container for " + scc.strProcessName);
	try {
		ServletContainer sc = new ServletContainer(scc);
		vecContainers.addElement(sc);
	} catch (Throwable t) {
		scc.logContext.write("? Unhandled execption during startup of " + scc.strClass, t);
	}
}
/**
 * creates a logfile. returns null if the creation failed
 */
private LogContext createLogContext(Properties pp) {
	LogContext log = null;
	String strPath = pp.getProperty("logfile_path");
	String strPre = pp.getProperty("logfile_praefix");
	String strSuf = pp.getProperty("logfile_suffix");
	String strFileFlt = pp.getProperty("logfile_options");
	String strScrnFlt = pp.getProperty("logscrn_options");
	String strDate = pp.getProperty("logfile_dateformat");
	String strTime = pp.getProperty("logfile_timeformat");
	boolean blnRemoveClassName = pp.getProperty("logfile_remove_classname").equals("1");
	//
	if (strScrnFlt.length() == 0)
		strScrnFlt = strFileFlt;
	//
	try {
		log = new LogContext(strPath, strPre, strSuf);
		log.setDateFormat(strDate);
		log.setTimeFormat(strTime);
		log.setFilter_File(strFileFlt);
		log.setFilter_Screen(strScrnFlt);
		log.setRemoveClassName(blnRemoveClassName);
		log.write("# Logfile opened");
	} catch (IOException e) {
		System.out.println("Cannot open logfile");
		System.out.println(e);
	}
	return log;
}
/**
 * creates an instance of a sessionmanager
 */
private SessionManager createSessionManager(String strConfigFile) {
	/**
	 * define some defaults
	 */
	Properties prpSessionManager = new Properties();
	prpSessionManager.put(SessionManager.SM_SESSIONTIMEOUT, "60");
	prpSessionManager.put(SessionManager.SM_CLEANUPCYCLE, "120");
	prpSessionManager.put("sessionmanager", "net.conacom.servletrunner.sessionmanager.MemorySessionManager");

	/**
	 * try to load the config
	 */
	InputStream in = null;
	try {
		in = new FileInputStream(strConfigFile);
		prpSessionManager.load(in);
	} catch (IOException e) {
		System.out.println("Error loading sessionmangerconfig: " + e.toString());
		System.out.println("Using defaults");
	} finally {
		Helper.close(in);
	}

	/**
	 * start an instance of the sessionmanager
	 */
	String strClass = prpSessionManager.getProperty("sessionmanager");
	logContext.write("# Starting Sessionmanager " + strClass);
	SessionManager smgr = null;
	try {
		Object[] objArgs = new Object[] {logContext, prpSessionManager};
		Class[] clsArgs = new Class[] {LogContext.class, Properties.class};
		java.lang.reflect.Constructor c = Class.forName(strClass).getConstructor(clsArgs);
		smgr = (SessionManager) c.newInstance(objArgs);
	} catch (ClassNotFoundException e) {
		logContext.write("? Class '" + strClass + "' not found");
		return null;
	} catch (InstantiationException e) {
		logContext.write("? Cannot create instance for '" + strClass + "'");
		return null;
	} catch (IllegalAccessException e) {
		logContext.write("? Access denied in class '" + strClass + "'");
		return null;
	} catch (NoSuchMethodException e) {
		logContext.write("? No matching constructor in class '" + strClass + "'");
		return null;
	} catch (java.lang.reflect.InvocationTargetException e) {
		logContext.write("? Error starting class '" + strClass + "'", e);
		return null;
	} catch (Throwable e) {
		logContext.write("? Unknown error while starting class '" + strClass + "'", e);
		return null;
	}
	//
	return smgr;
}
/**
 * 
 */
public void dispose() {
	blnDisposed = true;

	/**
	 * notify semaphore checker (its listening to blnDisposed)
	 */
	objNotify.lo_notify();
	thrdSemaCheck = null;
	objNotify.dispose();
	objNotify = null;

	/**
	 * dispose containers
	 */
	if (vecContainers != null) {
		logContext.write("D Disposing " + vecContainers.size() + " Containers");
		for (int i = 0; i < vecContainers.size(); i++) {
			try {
				((ServletContainer) vecContainers.elementAt(i)).dispose();
			} catch (Throwable t) {
				if (logContext != null)
					logContext.write("? Container .dispose()", t);
			}
		}
		vecContainers.removeAllElements();
		vecContainers = null;
		logContext.write("D Containers disposed");
	}
	// dispose sessionmanager
	if (smManager != null) {
		smManager.dispose();
		smManager = null;
	}
	// dispose logcontext
	if (logContext != null) {
		logContext.write("# Stopped --------------------------------------------------");
		logContext.dispose();
		logContext = null;
	}
	// remote control
	if (rcc != null) {
		rcc.dispose();
		rcc = null;
	}
	// run finalization ?
	if (prpRunnerConfig.getProperty("finalization").equals("1"))
		System.runFinalization();
}
/**
 * returns an instance of the RemoteControlConfig object
 */
public final static RemoteControlConfig getRemoteControlConfig() {
	return rcc;
}
/**
 * returns the version information for this servletrunner.
 */
public final static String getVersionInfo() {
	StringBuffer strb = new StringBuffer();
	strb.append("Sokrates LRWP Servletrunner/");
	strb.append(VERSION);
	strb.append(" ((C) 1999-2002 conacom GmbH http://runner.conacom.de)");
	return strb.toString();
}
/**
 * loads the runner configuration and initializes the servletrunner.
 */
private void init() {
	/**
	 * some preparations
	 */
	RunnerEventHandler.addListener(this);
	blnDisposed = false;
	Thread.currentThread().setName("Sokrates");

	/**
	 * some defaults for runner.ini
	 */
	prpRunnerConfig = new Properties();
	prpRunnerConfig.put("default_host", "127.0.0.1"); // default host to connect
	prpRunnerConfig.put("default_port", "81"); // default port to connect 
	prpRunnerConfig.put("logfile_path", ""); // path to the logfiles
	prpRunnerConfig.put("logfile_praefix", "srv_"); // praefix for the logfiles
	prpRunnerConfig.put("logfile_suffix", ".log"); // suffix for the logfiles
	prpRunnerConfig.put("logfile_options", "D#!?S"); // Log options for file. D=Debug, #=Info, !=Warning, ?=Errors, S=output from the servlet
	prpRunnerConfig.put("logscrn_options", ""); // Log options for screen
	prpRunnerConfig.put("logfile_dateformat", "dd.MM.yyyy"); // dateformat to use
	prpRunnerConfig.put("logfile_timeformat", "HH:mm:ss"); // timeformat to use
	prpRunnerConfig.put("logfile_remove_classname", "0"); // remove classname from servlet-log
	prpRunnerConfig.put("sessionmanagerconfig", ""); // configurationfile for the sessionmanager
	prpRunnerConfig.put("finalization", "0"); // call System.runFinalization() in dispose
	prpRunnerConfig.put("runfinalizersonexit", "0"); // System.runFinalizersOnExit(true) in dispose()
	prpRunnerConfig.put("locale", ""); // the locale to use
	prpRunnerConfig.put("semaphorecheck", "1000"); // time to wait between checking the semaphores
	prpRunnerConfig.put("goodbye_message", "Sie haben ein einfaches Programm sehr glcklich gemacht");
	prpRunnerConfig.put("tempdir_root", "tempfiles"); // temporary directory

	/**
	 * load runner.ini
	 */
	InputStream in = null;
	try {
		in = new FileInputStream(CONFIGFILE);
		prpRunnerConfig.load(in);
	} catch (IOException e) {
		System.out.println("Error loading " + CONFIGFILE + ": " + e.getMessage());
		dispose();
		return;
	} finally {
		Helper.close(in);
	}

	/**
	 * try to open the logfile
	 */
	logContext = createLogContext(prpRunnerConfig);
	if (logContext == null) {
		dispose();
		return;
	}

	/**
	 * start sempaphore check first to have a chance for restart or shutdown.
	 * we hope that nothing went wrong until now.
	 */
	objNotify = new LockObject();
	thrdSemaCheck = new Thread(this, "Sem watchdog");
	thrdSemaCheck.start();

	/**
	 * log some useful informations
	 */
	Properties p = System.getProperties();
	String s = p.getProperty("os.name");
	s = s + " v" + p.getProperty("os.version");
	s = s + " (" + p.getProperty("os.arch") + ")";
	s = "Sokrates v" + VERSION + " " + s;
	logContext.write("# " + s);

	/**
	 * create the tempdir
	 */
	flTempRoot = new File(prpRunnerConfig.getProperty("tempdir_root"));
	try {
		flTempRoot.mkdirs();
	} catch (Throwable t) {
		logContext.write("? Cannot create tempdir (" + flTempRoot.toString() + "), aborting", t);
		dispose();
		return;
	}

	/**
	 * set the locale
	 */
	String strLocale = prpRunnerConfig.getProperty("locale");
	if (strLocale.length() != 0) {
		StringTokenizer st = new StringTokenizer(strLocale, ",");
		String s1 = st.nextToken();
		String s2 = st.nextToken();
		Locale loc = null;
		if (st.hasMoreTokens())
			loc = new Locale(s1, s2, st.nextToken());
		else
			loc = new Locale(s1, s2);
		Locale.setDefault(loc);
		logContext.write("# Set Locale to " + loc.toString());
	}

	/**
	 * try to start the sessionmanager
	 */
	smManager = createSessionManager(prpRunnerConfig.getProperty("sessionmanagerconfig"));
	if (smManager == null) {
		logContext.write("? Cannot start sessionmanager, aborting");
		dispose();
		return;
	}

	/**
	 * now load the configuration for the servlets 
	 * servlet=c:\www\vhosts\www.conacom.de\servlet\servlet.cfg
	 * create a ServletContainer per config-file
	 * we cannot use Properties because we have multiple lines with "servlet_cfg"
	 */
	vecContainers = new Vector();
	BufferedReader br = null;
	try {
		logContext.write("# Reading configuration");
		br = new BufferedReader(new FileReader(CONFIGFILE));
		String strBuffer = null;
		while ((strBuffer = br.readLine()) != null) {
			strBuffer = strBuffer.trim();
			int pp = strBuffer.indexOf("=");
			if (pp == -1)
				continue;
			String strKey = strBuffer.substring(0, pp).trim();
			String strFile = strBuffer.substring(pp + 1).trim();
			if (strKey.equalsIgnoreCase("servlet_cfg")) {
				// create a servletcontainer from the configuration in strFile
				createContainer(strFile);
			}
		}
		br.close();
	} catch (IOException e) {
		Helper.close(br);
		logContext.write("? Cannot read servlet configuration.", e);
		dispose();
	}

	/**
	 * remote control
	 */
	rcc = new RemoteControlConfig();
	rcc.runner = this;
	rcc.logContext = logContext;
	rcc.vecContainers = vecContainers;
	rcc.smgr = smManager;

	/**
	 * were done
	 */
	logContext.write("# Started -----------------------------");
}
/**
 * command line argument is different ini file (default: runner.ini)
 */
public static void main(String args[]) {
	// commandline ?
	if (args.length > 0) {
		Runner.CONFIGFILE = args[0];
	}
	//
	Runner r = null;
	try {
		r = new Runner();
	} catch (Throwable t) {
		if (r != null && r.logContext != null) {
			r.logContext.write("? Unhandled exception in .init():", t);
		} else {
			System.out.println("Unhandled exception in .init():");
			t.printStackTrace();
		}
	}
}
/**
 * checks file semaphores for shutdown and restart
 */
public void run() {
	logContext.write("D Starting runner watchdog");
	//
	File f = null;

	// check for old files
	f = new File("shutdown");
	if (f.exists())
		f.delete();
	f = new File("reload");
	if (f.exists())
		f.delete();

	// get the delay
	long lngDelay = 1000l;
	try {
		lngDelay = Long.parseLong(prpRunnerConfig.getProperty("semaphorecheck"));
	} catch (NumberFormatException e) {
		logContext.write("? Unable to parse value of 'semaphorecheck'");
	}

	//
	try {
		while (!blnDisposed) {
			f = new File("shutdown");
			if (f.exists()) {
				f.delete();
				RunnerEventHandler.postEvent(new RunnerEvent(this, RunnerEvent.RE_SHUTDOWN));
				return;
			}
			f = new File("restart");
			if (f.exists()) {
				f.delete();
				RunnerEventHandler.postEvent(new RunnerEvent(this, RunnerEvent.RE_RESTART));
				return;
			}
			objNotify.lo_wait(lngDelay);
		}
	} catch (Throwable t) {
		logContext.write("? Unhandled exception in runner watchdog", t);
	}
	logContext.write("? runner watchdog exit");
}
/**
 * 
 */
public void runnerEvent(RunnerEvent e) {
	logContext.write("# Got Event " + e.toString());
	// shutdown ?
	switch (e.getID()) {
		case e.RE_SHUTDOWN :
			dispose();
			if (prpRunnerConfig.getProperty("runfinalizersonexit").equals("1"))
				System.runFinalizersOnExit(true);
			else
				System.runFinalizersOnExit(false);
			// bye	
			System.out.println(prpRunnerConfig.getProperty("goodbye_message"));
			System.exit(0);
			break;
		case e.RE_RESTART :
			dispose();
			init();
			break;
	}
}
}