package net.conacom.servletrunner.main;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.util.*;
import net.conacom.servletrunner.util.*;
import net.conacom.servletrunner.sessionmanager.*;
/**
 * contains the configuration for a bunch of servlets in a container
 *
 * 07.05.02 ho: fixed a problem in dispose when the classloader wasn't initialized
 * 28.04.02 ho: fixed bug in .dispose()
 * 19.04.02 ho: retrydelay/retries
 * 04.04.02 ho: removed version
 * 19.03.02 ho: creation
 */
public class ServletContainerConfig extends Application {
	// identifier (praefix or .suffix) which identifies an URL as a servlet
	// its something like "servlet" or ".srv"
	public String strServletIdentifier = "";
	// classname of the servlet
	public String strClass = "";
	// virtual host
	public String strVirtualHost = "";
	// number of instances to create
	public int intNbOfInstances = 0;
	// a log context
	public LogContext logContext = null;
	// flag: we're using our own logger instead the logger of the Runner
	boolean blnOwnLogger = false;
	// webserver to connect
	public String strLRWP_Host = null;
	// port to connect
	public String strLRWP_Port = null;
	// a property with additional configuration options
	public Properties prpConfig = null;
	// a hashtable where all instances of a container can share informations
	// this table is mainly used in the ServletContext as attribute container
	public Hashtable hshAttributes = null;
	// this object is used in the ServletInstance to notify the container about a dead thread
	public LockObject objNotifyDeath = new LockObject();
	// default size of the request outputbuffer
	public int intOutputBufferSize = 0;
	// statistics
	public ServletContainerStatistics stats = new ServletContainerStatistics();
	// the threadgroup where the instances of this container are running (just to keep our sheeps together)
	public ThreadGroup thrdgInstances = null;
	// the session-storage manager (all instances and containers are sharing the same manager !)
	// ho: we need additional programming because we like to group containers with accessrights
	public static SessionManager smManager = null;
	// the processname from the configuration
	public String strProcessName = null;
	// the classloader used by this container
	public RunnerClassLoader clLoader = null;
	// use cookies to store the session-id ? TRUE=use cookies, FALSE=use URL
	public boolean blnUseCookieForSession = true;
	// lrwp retries
	public int intLRWP_Retries = -1;
	// lrwp retry delay
	public long lngLRWP_RetryDelay = -1;
	// time to wait for constructor completion
	public long lngMonitor_instance = 0;
	// time to wait for .init(..) completion
	public long lngMonitor_init = 0;
	// time to wait for .service(..) completion
	public long lngMonitor_service = 0;
	// time to wait for .init(..) completion
	public long lngMonitor_destroy = 0;
	// name of the configuration file
	public String strConfigFile = null;
	// allow reverse DNS for HttpServletRequesst#getRemoteHost()
	public boolean blnRDNS = false;
	// imported servlets (import=)
	public Hashtable hshImports = null;
/**
 *
 */
public ServletContainerConfig() {
	super();
}
/**
 * 
 */
public void dispose() {
	if (objNotifyDeath != null) {
		objNotifyDeath.dispose();
		objNotifyDeath = null;
	}
	blnDisposed = true;
	strServletIdentifier = null;
	String strClass = null;
	String strVirtualHost = null;
	strLRWP_Host = null;
	strLRWP_Port = null;
	if (prpConfig != null) {
		prpConfig.clear();
		prpConfig = null;
	}
	if (hshAttributes != null) {
		hshAttributes.clear();
		hshAttributes = null;
	}
	if (clLoader == null) {
		clLoader.dispose();
		clLoader = null;
	}
	// should be the last action here
	if (blnOwnLogger && logContext != null) {
		logContext.dispose();
		logContext = null;
	}
}
}