package net.conacom.servletrunner.main;

/**
 * Copyright 1999,2002 by conacom GmbH, http://www.conacom.de
 * Copyright 1999-2002 Bernd Hohmann, hohmann@harddiskcafe.de
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import net.conacom.servletrunner.impl.*;
import net.conacom.servletrunner.util.*;
/**
 * contains a servlet instance. created by the ServletContainer
 * performs all servletcommuication.
 *
 * 05.07.02 ho: fixed race condition on blnIsRunning
 * 25.06.02 ho: queued startup is default now. using LockObject for synchronizing run() against .init()
 *				session cookie age =-1 (browser session only)
 * 10.06.02 ho: procesError(..): uses rootcause of error now
 * 23.05.02 ho: SessionCookie.maxAge -> SessionManger.getDefaultSessionTimeout()
 * 22.05.02 ho: changed errormessage output
 * 16.05.02 ho: monitored execution
 * 10.04.02 ho: speedups
 * 25.03.02 ho: requestbody as stream instead of bytestring
 * 24.03.02 ho: other method of creating an servletinstance
 * 19.03.02 ho: creation
 */
public class ServletInstance extends Application implements Runnable {
	// the loaded servlet
	private Servlet servlet = null;
	// the connection to the webserver
	private LRWP_Connection lrwp_con = null;
	// configuration for all instances in this container
	private ServletContainerConfig scc = null;
	// the thread where all requests are handled
	private Thread thrd = null;
	// ID of this instance (dummy [1/6]
	private String strInstanceID = null;
	// flag: we passed .init() and .run() is started
	private boolean blnIsRunning = false;
	// javax.servlet
	private ServletContext_Impl srvlContext = null;
	private ServletConfig_Impl srvlConfig = null;
	private HttpServletRequest_Impl srvlRequest = null;
	private HttpServletResponse_Impl srvlResponse = null;
	// statistics
	private volatile static int intParallelRequests = 0;
	// monitored execution
	private MonitoredExecution monitor = null;
	// lock object for startup
	private LockObject objLock = new LockObject();
	// cache for Servlets used by a RequestDispatcher
	private Hashtable hshServletCache = null;
/**
 *
 */
public ServletInstance(ServletContainerConfig scc, String strInstanceID) {
	super();
	this.scc = scc;
	this.strInstanceID = strInstanceID;
	// we need to save and reset the threadname for the logger
	String strTmp = Thread.currentThread().getName();
	Thread.currentThread().setName(strInstanceID);
	try {
		init();
	} catch (Throwable t) {
		t.fillInStackTrace();
		throw new RunnerException(t);
	} finally {
		Thread.currentThread().setName(strTmp);
	}
}
/**
 * creates the instance. public because its called from a different class
 */
public Servlet createInstance(String strClass) {
	try {
		scc.logContext.write("D Loading class " + strClass);
		Object[] objArgs = new Object[] {};
		Class[] clsArgs = new Class[] {};
		Class cls = scc.clLoader.loadClass(strClass);
		java.lang.reflect.Constructor c = cls.getConstructor(clsArgs);
		scc.logContext.write("D Creating instance for " + strClass);
		Object objServlet = c.newInstance(objArgs);
		if (!(objServlet instanceof Servlet)) {
			scc.logContext.write("? Class " + strClass + " isn't a Servlet-Class");
			return null;
		}
		return (Servlet) objServlet;
	} catch (ClassNotFoundException e) {
		scc.logContext.write("? Class " + strClass + " cannot be loaded");
		return null;
	} catch (InstantiationException e) {
		scc.logContext.write("? Cannot create instance of " + strClass + ":" + e.getMessage());
		return null;
	} catch (IllegalAccessException e) {
		scc.logContext.write("? Cannot create instance of " + strClass + ": " + e.getMessage());
		return null;
	} catch (NoSuchMethodException e) {
		scc.logContext.write("? No matching constructor in " + strClass + ": " + e.getMessage());
		return null;
	} catch (java.lang.reflect.InvocationTargetException e) {
		scc.logContext.write("? Error while calling constructor of " + strClass + ": " + e.getTargetException().getMessage());
		return null;
	}
}
/**
 * 
 */
public void dispose() {
	blnDisposed = true;
	blnIsRunning = false;
	// stop servlets in requestdispatcher
	if (hshServletCache != null) {
		for (Enumeration e = hshServletCache.elements(); e.hasMoreElements();) {
			instanceStop((Servlet) e.nextElement());
		}
		hshServletCache = null;
	}
	// stop servlet
	if (servlet != null) {
		instanceStop(servlet);
		servlet = null;
	}
	//
	if (srvlConfig != null) {
		srvlConfig.dispose();
		srvlConfig = null;
	}
	//
	if (srvlContext != null) {
		srvlContext.dispose();
		srvlContext = null;
	}
	//
	if (srvlRequest != null) {
		srvlRequest.dispose();
		srvlRequest = null;
	}
	if (srvlResponse != null) {
		srvlResponse.dispose();
		srvlResponse = null;
	}
	if (monitor != null) {
		monitor.dispose();
		monitor = null;
	}
	// will kill the connection and stop the thread
	if (lrwp_con != null) {
		lrwp_con.dispose();
		lrwp_con = null;
	}
	if (objLock != null) {
		objLock.dispose();
		objLock = null;
	}
}
/**
 * returns a request dispatcher
 * strResource can be 
 */
public RequestDispatcher getRequestDispatcher(String strResource, boolean blnNamedResource) {
	/**
	 * get classname from "import=" statements of container configuration
	 * 1) if this is called from getNamedDispatcher(), the resorce is an alias name
	 * 2) if this is called from getRequestDispatcher(), the resource is a classname
	 */
	// strip instance name
	if (strResource.startsWith(scc.strServletIdentifier + "/"))
		strResource = strResource.substring(scc.strServletIdentifier.length() + 1);
	// strip query arguments
	String strQuery[] = null;
	int pp = strResource.indexOf("?");
	if (pp != -1) {
		strQuery = new String[] {strResource.substring(pp + 1)};
		strResource = strResource.substring(0, pp);
	}
	/**
	 * check if resource is configured in container config, get class for alias too
	 */
	String strClass = null;
	if (blnNamedResource) {
		strClass = (String) scc.hshImports.get(strResource); // resolve alias
	} else {
		strClass = scc.hshImports.contains(strResource) ? strClass = strResource : null; // test only if in import list 
	}
	if (strClass == null)
		throw new RunnerException("Resource " + strResource + " not in 'import' statement");
	/**
	 * check if class is in cache, create an instance otherwise.
	 * we can do this without a time monitor because this method is
	 * called in the monitored .service(..) environment of an existing servlet
	 */
	Servlet srvl = (Servlet) hshServletCache.get(strClass);
	if (srvl == null) {
		srvl = createInstance(strClass);
		if (srvl == null)
			return null;
		try {
			srvl.init(srvlConfig);
		} catch (Throwable t1) {
			scc.logContext.write("? Error while calling .init(...); will .destroy()", t1);
			try {
				srvl.destroy();
			} catch (Throwable t2) {
				scc.logContext.write("? Error while calling .destroy(...)", t2);
			}
			return null;
		}
		hshServletCache.put(strClass, srvl);
	}
	/**
	 * return request dispatcher
	 */
	return new RequestDispatcher_Impl(srvl, strQuery, scc, this);
}
/**
 * initializes the instance.
 *
 * errorhandling: local (write to logfile, .dispose() and return)
 */
private void init() {
	/**
	 * creates the execution monitor	 
	 */
	monitor = new MonitoredExecution();

	/**
	 * creates the config for the LRWP connection
	 */
	LRWP_ConnectionConfig cfg = new LRWP_ConnectionConfig();
	cfg.strLRWP_Version = "2.0";
	cfg.strLRWP_Host = scc.strLRWP_Host;
	cfg.strAccessKey = null;
	cfg.strServletIdentifier = scc.strServletIdentifier;
	cfg.strVirtualHost = scc.strVirtualHost;
	cfg.logContext = scc.logContext;
	try {
		cfg.intLRWP_Port = Integer.parseInt(scc.strLRWP_Port);
	} catch (NumberFormatException e) {
		scc.logContext.write("? Unparseable Port:" + scc.strLRWP_Port);
		dispose();
		return;
	}
	cfg.intRetries = scc.intLRWP_Retries;
	cfg.lngRetryWait = scc.lngLRWP_RetryDelay;

	/**
	 * connects to the webserver
	 */
	try {
		scc.logContext.write("D Connecting to LRWP-Host " + cfg.strLRWP_Host);
		lrwp_con = new LRWP_Connection(cfg);
		scc.logContext.write("D connected " + cfg.strLRWP_Host);
	} catch (RunnerException e) {
		scc.logContext.write("? Cannot connect to webserver " + e.getMessage());
		dispose();
		return;
	}

	/**
	 * create the servlet environment
	 */
	srvlContext = new ServletContext_Impl(scc, this);
	srvlConfig = new ServletConfig_Impl(srvlContext, scc.prpConfig);
	// this objects will be recycled on every request
	srvlRequest = new HttpServletRequest_Impl(scc, this);
	srvlResponse = new HttpServletResponse_Impl(scc);

	/**
	 * caches for RequestDispatcher
	 */
	hshServletCache = new Hashtable();

	/**
	 * creates and initializes the servlet
	 */
	servlet = instanceStart(scc.strClass, srvlConfig);
	if (servlet == null) {
		dispose();
		return;
	}

	/**
	 * starts the servlet thread
	 */
	blnIsRunning = true;
	scc.logContext.write("D Starting Servlet instance");
	thrd = new Thread(scc.thrdgInstances, this, strInstanceID);
	thrd.start();

	/**
	 * wait for the thread coming up
	 */
	objLock.lo_wait(0);
}
/**
 * 1) creates a fresh servlet instance
 * 2) calls .init() on the servlet
 *
 * returns NULL if something failed
 */
private Servlet instanceStart(String strClass, ServletConfig scConfig) {
	/**
	 * creates an servletinstance
	 */
	Object objResult = monitor.execute(this, "createInstance", new Object[] {strClass}, new Class[] {String.class}, scc.lngMonitor_instance);
	if (monitor.hadTimeout()) {
		scc.logContext.write("? .createInstance() didn't finished in time !");
		return null;
	}
	if (monitor.getThrowable() != null) {
		scc.logContext.write("? Error while calling .createInstance()", monitor.getThrowable());
		return null;
	}
	if (objResult == null) {
		return null;
	}

	/**
	 * try to call servlet-init
	 */
	scc.logContext.write("D Calling init(...) of " + strClass);
	monitor.execute((Servlet) objResult, "init", new Object[] {scConfig}, new Class[] {ServletConfig.class}, scc.lngMonitor_init);
	if (monitor.hadTimeout()) {
		scc.logContext.write("? .init(..) didn't finished in time !");
		return null;
	}
	if (monitor.getThrowable() != null) {
		scc.logContext.write("? Error while calling .init(...)", monitor.getThrowable());
		return null;
	}

	// done
	return (Servlet) objResult;
}
/**
 * stops a servlet instance
 */
private void instanceStop(Servlet srvl) {
	scc.logContext.write("D Calling destroy");
	monitor.execute(srvl, "destroy", new Object[] {}, new Class[] {}, scc.lngMonitor_destroy);
	if (monitor.hadTimeout()) {
		scc.logContext.write("? destroy() didn't finished in time");
	}
	if (monitor.getThrowable() != null) {
		scc.logContext.write("? Error while destroying the servlet", monitor.getThrowable());
	}
	scc.logContext.write("D Destroyed");
}
/**
 * definition: the thread is defined at the end of .init() and the thread is still running.
 */
public boolean isRunning() {
	return thrd != null && blnIsRunning == true;
}
/**
 * dump an error to HTML
 */
private void processError(Throwable t) {
	// keep the original message
	String strMsg = t.getMessage();
	// get the root cause
	while (t instanceof ServletException) {
		t = ((ServletException) t).getRootCause();
	}
	scc.logContext.write(strMsg, t);
	//
	ByteString bs = new ByteString();
	bs.append("<html>\n");
	bs.append(" <head>\n");
	bs.append("  <title>Error page</title>\n");
	bs.append(" </head>\n");
	bs.append(" <body>\n");
	bs.append("  An error has occured: " + t.getMessage() + "<br>");
	bs.append("  <hr>");
	try {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		t.printStackTrace(pw);
		BufferedReader br = new BufferedReader(new StringReader(sw.toString()));
		String strBuffer = null;
		while ((strBuffer = br.readLine()) != null) {
			if (strBuffer.length() != 0) {
				bs.append("   ");
				bs.append(strBuffer);
				bs.append("<br>\n");
			}
		}
	} catch (IOException e) {
		// never happens because we're I/O'ing strings only
	}
	bs.append("  <hr>");
	bs.append("[end of stackdump]<br>\n");
	bs.append("<i>" + Runner.getVersionInfo() + "</i>\n");
	bs.append(" </body>\n");
	bs.append("</html>\n");
	lrwp_con.dataSend(bs);
}
/**
 * 
 */
public void run() {
	//
	objLock.lo_notify();
	scc.logContext.write("D Servlet instance started");
	//
	while (blnIsRunning) {
		/**
		 * wait for a request from the webserver
		 */
		long lngRequestTime = 0;
		long lngServletTime = 0;
		try {
			scc.logContext.write("D Waiting for request");
			lrwp_con.dataRetrieve();
			// disposed in the meantime ?
			if (blnDisposed)
				break;
			lngRequestTime = System.currentTimeMillis();
		} catch (Throwable t1) {
			scc.logContext.write("? Error in lrwp_con.dataRetrieve()", t1);
			lrwp_con.dispose();
			scc.objNotifyDeath.lo_notify();
			break;
		}

		/**
		 * check if we have been disconnected in the meantime.
		 * in this case notify the container and try to restart the instance
		 * if we're currently disposing, we do nothing
		 */
		if (!lrwp_con.isConnected()) {
			if (!blnDisposed) {
				scc.logContext.write("! Disconnected from webserver, trying to restart instance");
				dispose();
				synchronized (scc.objNotifyDeath) {
					scc.objNotifyDeath.notify();
				}
			}
			break;
		}

		/**
		 * handle the request
		 */
		// assemble parameters
		Hashtable hshHeaders = lrwp_con.getHeader();
		Hashtable hshParameters = lrwp_con.getParameters();
		ServletInputStream sisBody = lrwp_con.getRequestBody();
		srvlRequest._initObject(hshHeaders, hshParameters, sisBody);
		srvlContext._initObject(srvlRequest, srvlConfig);
		srvlResponse._initObject(srvlRequest);
		//
		boolean blnHasError = false;
		scc.logContext.write("# Got request from " + srvlRequest.getRemoteHost() + "/" + srvlRequest.getRemoteAddr());
		intParallelRequests++;
		// call .service(...) under control of the monitor
		Object objParam[] = new Object[] {srvlRequest, srvlResponse};
		Class clsParam[] = new Class[] {ServletRequest.class, ServletResponse.class};
		lngServletTime = System.currentTimeMillis();
		monitor.execute(servlet, "service", objParam, clsParam, scc.lngMonitor_service);
		if (monitor.hadTimeout()) {
			processError(new RunnerException(".service(...) didn't finish processing in time"));
			scc.stats.lngNbOfErrors++;
			blnHasError = true;
		}
		if (monitor.getThrowable() != null) {
			processError(monitor.getThrowable());
			scc.stats.lngNbOfErrors++;
			blnHasError = true;
		}
		lngServletTime = System.currentTimeMillis() - lngServletTime;

		/**
		 * if we had an error, we don't store sessiondata or send back data
		 */
		if (!blnHasError) {
			/**
			 * store sessiondata
			 */
			HttpSession_Impl hshData = srvlRequest._getHttpSession();
			if (hshData != null) {
				if (scc.blnUseCookieForSession) {
					Cookie coo = new Cookie(HttpSession_Impl.SESSION_IDENTIFIER, hshData.getId());
					coo.setMaxAge(-1);
					srvlResponse.addCookie(coo);
				}
				scc.smManager.putHttpSession(hshData);
			}

			/**
			 * send the data
			 */
			ByteString bsResponse = srvlResponse._getResponseData();
			lrwp_con.dataSend(bsResponse);
			scc.logContext.write("D Finished request");

			/**
			 * statistics
			 */
			synchronized (scc.stats) {
				scc.stats.lngNbOfRequests++;
				if (intParallelRequests == scc.intNbOfInstances)
					scc.stats.lngNbOfPeakRequests++;
				scc.stats.lngTotalTimeInRequest += (System.currentTimeMillis() - lngRequestTime);
				scc.stats.lngTotalTimeInServlet += lngServletTime;
				scc.stats.intMinBufferSize = Math.min(bsResponse.getLength(), scc.stats.intMinBufferSize);
				scc.stats.intMaxBufferSize = Math.max(bsResponse.getLength(), scc.stats.intMaxBufferSize);
				double d = ((double) bsResponse.getLength()) / 1024d;
				for (int i = 0; i < scc.stats.dblBufferGrid.length; i++) {
					if (scc.stats.dblBufferGrid[i] > d) {
						++scc.stats.lngBufferCnt[i];
						break;
					}
				}
			}
		} // blnHasError/

		/**
		 * cleanup
		 */
		intParallelRequests--;
	}
	blnIsRunning = false;
}
}