package net.conacom.servletrunner.sessionmanager;

/**
 * Copyright 1999-2002 by conacom GmbH
 * Wilhelmshoeher Strasse 202, 60389 Frankfurt/Main, Germany
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.util.*;
import javax.servlet.http.*;
import net.conacom.servletrunner.util.*;
import net.conacom.servletrunner.impl.*;
import net.conacom.servletrunner.main.*;
/**
 * a sessionmanger who stores all values in memory
 *
 * 08.04.02 ho: sessioncounter isnt static anymore
 * 23.03.02 ho: creation
 */
public class MemorySessionManager extends SessionManager {
	private Hashtable hshData = null;
	private int intSessionCount = 0;
/**
 * MemorySessionStorage - Konstruktorkommentar.
 * @param log net.conacom.servletrunner.main.LogContext
 * @param tfTime net.conacom.servletrunner.util.TimeFactory
 * @param lngSessionTimeout long
 * @param lngCleanupIdleTime long
 */
public MemorySessionManager(LogContext log, Properties prp) {
	super(log, prp);
	hshData = new Hashtable();
}
/**
 * cleanup of outdated sessions
 */
protected void cleanup() {
	synchronized (getLock()) {
		log.write("# Starting cleanup");
		int intCntCleaned = 0;
		int intCntTotal = 0;
		for (Enumeration e = hshData.elements(); e.hasMoreElements();) {
			intCntTotal++;
			SessionData sd = (SessionData) e.nextElement();
			if (sd.hshValues == null || !sd.isValidSession()) {
				hshData.remove(sd.strID);
				intCntCleaned++;
			}
		}
		log.write("# Removed " + intCntCleaned + " entries of " + intCntTotal);
	}
}
/**
 * 
 */
public SessionData createSession(HttpServletRequest req) {
	long lngTime = TimeFactory.getTime();
	//
	SessionData sc = new SessionData();
	sc.blnIsNewSession = true;
	sc.hshValues = new Hashtable();
	sc.lngCreationTime = lngTime;
	sc.lngLastAccessedTime = lngTime;
	sc.lngMaxInactiveInterval = lngSessionTimeout;
	// maybe we shall encrypt the IP also into the ID
	sc.strID = "ID" + Long.toHexString(lngTime) + "-" + Integer.toString(++intSessionCount);
	//
	return sc;
}
/**
 * gets a session
 */
protected SessionData getSession(String strID) {
	synchronized (getLock()) {
		return (SessionData) hshData.get(strID);
	}
}
/**
 * 
 * @return java.lang.String
 */
public String getStatistics() {
	synchronized (getLock()) {
		StringBuffer strb = new StringBuffer();
		strb.append("Entries total: ");
		strb.append(hshData.size());
		strb.append(", valid entries: ");
		int intValid = 0;
		for (Enumeration e = hshData.elements(); e.hasMoreElements();) {
			SessionData sd = (SessionData) e.nextElement();
			if (sd.hshValues != null && sd.isValidSession()) {
				intValid++;
			}
		}
		strb.append(intValid);
		return strb.toString();
	}
}
/**
 * session for ID is valid ?
 */
public boolean isValid(String strID) {
	if (strID == null)
		return false;
	//
	synchronized (getLock()) {
		SessionData sd = (SessionData) hshData.get(strID);
		return sd != null && sd.isValidSession();
	}
}
/**
 * save session
 */
protected void putSession(SessionData sd) {
	synchronized (getLock()) {
		hshData.put(sd.strID, sd);
	}
}
}