package net.conacom.servletrunner.util;

/**
 * Copyright 1999-2002 by conacom GmbH
 * Wilhelmshoeher Strasse 202, 60389 Frankfurt/Main, Germany
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
/**
 * 25.05.02 ho: clone of Vector which supports a Key/Data pair
 */
public class DataVector {
	private Object objKeys[] = null;
	private Object objData[] = null;
	private int intElementCount = -1;
	private int intCapacityIncrement = -1;
public DataVector() {
	this(10);
}
public DataVector(int intInitialCapacity) {
	this(intInitialCapacity, 0);
}
public DataVector(int intInitialCapacity, int intCapacityIncrement) {
	this.objKeys = new Object[intInitialCapacity];
	this.objData = new Object[intInitialCapacity];
	this.intElementCount = 0;
	this.intCapacityIncrement = intCapacityIncrement;
}
public void addKeyData(Object objNewKey, Object objNewData) {
	int newcount = intElementCount + 1;
	if (newcount > objKeys.length) {
		ensureCapacityHelper(newcount);
	}
	objKeys[intElementCount] = objNewKey;
	objData[intElementCount] = objNewData;
	intElementCount++;
}
/**
 * 
 */
public void clear() {
	for (int i = 0; i < intElementCount; i++) {
		objKeys[i] = null;
		objData[i] = null;
	}
	intElementCount = 0;
}
public boolean containsKey(Object objSearchKey) {
	return indexOfKey(objSearchKey) != -1;
}
public Object dataAt(int idx) {
	if (idx >= intElementCount) {
		throw new ArrayIndexOutOfBoundsException(idx + " >= " + intElementCount);
	}
	try {
		return objData[idx];
	} catch (ArrayIndexOutOfBoundsException e) {
		throw new ArrayIndexOutOfBoundsException(idx + " < 0");
	}
}
public Object dataForKey(Object objSearchKey) {
	int i = indexOfKey(objSearchKey);
	return i != -1 ? dataAt(i) : null;
}
private void ensureCapacityHelper(int intMinCapacity) {
	int oldCapacity = objKeys.length;
	Object oldKeys[] = objKeys;
	Object oldData[] = objData;
	int newCapacity = (intCapacityIncrement > 0) ? (oldCapacity + intCapacityIncrement) : (oldCapacity * 2);
	if (newCapacity < intMinCapacity) {
		newCapacity = intMinCapacity;
	}
	objKeys = new Object[newCapacity];
	System.arraycopy(oldKeys, 0, objKeys, 0, intElementCount);
	objData = new Object[newCapacity];
	System.arraycopy(oldData, 0, objData, 0, intElementCount);
}
/**
 * 
 * @return int
 * @param objKey java.lang.Object
 */
public int indexOfKey(Object objSearchKey) {
	for (int i = 0; i < intElementCount; i++) {
		if (objKeys[i].equals(objSearchKey))
			return i;
	}
	return -1;
}
public void insertOrReplaceDataForKey(Object objSearchKey, Object objNewData) {
	int i = indexOfKey(objSearchKey);
	if (i == -1)
		addKeyData(objSearchKey, objNewData);
	else
		objData[i] = objNewData;
}
public Object keyAt(int idx) {
	if (idx >= intElementCount) {
		throw new ArrayIndexOutOfBoundsException(idx + " >= " + intElementCount);
	}
	try {
		return objKeys[idx];
	} catch (ArrayIndexOutOfBoundsException e) {
		throw new ArrayIndexOutOfBoundsException(idx + " < 0");
	}
}
public int size() {
	return intElementCount;
}
}