package net.conacom.servletrunner.util;

/**
 * Copyright 1999-2002 by conacom GmbH
 * Wilhelmshoeher Strasse 202, 60389 Frankfurt/Main, Germany
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import javax.servlet.*;
/**
 * static helper methods like closing and standard exceptions
 *
 * 25.06.02 ho: stringEnumerator, listEnumerator
 * 19.05.02 ho: headerkey2cgikey, cgikey2headerkey
 * 13.05.02 ho: platformPath
 * 28.04.02 ho: getDirectory
 * 08.04.02 ho: sortvector
 * 03.04.02 ho: close serversocket
 * 26.03.02 ho: close zip
 * 24.03.02 ho: flushing writer
 * 19.03.02 ho: creation
 */
public class Helper {
	private static final int CASEDIFF = ('a' - 'A');
/**
 * don't create instances
 */
private Helper() {
	super();
}
/**
 * translates a string like "USER_AGENT" to "User-Agent"
 * needed for getAttribute() and others
 */
public final static String cgikey2headerkey(String strIn) {
	strIn = Helper.toLowerCase(strIn);
	strIn = strIn.replace('_', '-');
	StringBuffer strb = new StringBuffer(strIn);
	char chLast = '-';
	for (int i = 0; i < strb.length(); i++) {
		char ch = strb.charAt(i);
		if (chLast == '-')
			strb.setCharAt(i, (char) (ch - CASEDIFF));
		chLast = ch;
	}
	return strb.toString();
}
/**
* tests if the given String matches the template*
* templates look like "192.*.3.*"
*/
public static final boolean checkString(String s, String templ) {
	s = toLowerCase(s);
	templ = toLowerCase(templ);
	int wildPos = templ.indexOf('*');
	if (wildPos == -1) {
		return s.equals(templ);
	}
	final int templLen = templ.length();
	final int strLen = s.length();
	int templIndex = 0; // the current position in the template
	int strIndex = 0; // the current position in the string
	boolean wildCard = false; // if false, use regionMatches
	while (templIndex < templLen) {
		// get the next part
		if (templ.charAt(templIndex) == '*') {
			wildCard = true;
			templIndex++;
			continue;
		}
		int nextWild = templ.indexOf('*', templIndex);
		// no next wildcard? take the length
		if (nextWild == -1) {
			nextWild = templLen;
		}
		String templPart = templ.substring(templIndex, nextWild);
		templIndex = nextWild;
		if (wildCard) {
			int pos = s.indexOf(templPart);
			if (pos == -1)
				return false;
			strIndex = pos + templPart.length();
		} else {
			if (!s.regionMatches(false, 0, templPart, 0, templPart.length()))
				return false;
			strIndex += templPart.length();
		}
		wildCard = false;
	}
	return wildCard ? true : strIndex == strLen;
}
/**
 * closes an object
 */
public final static boolean close(Object o) {
	if (o == null)
		return true;
	//
	try {
		if (o instanceof InputStream) {
			((InputStream) o).close();
			return true;
		}
		if (o instanceof OutputStream) {
			((OutputStream) o).close();
			return true;
		}
		if (o instanceof Writer) {
			((Writer) o).close();
			return true;
		}
		if (o instanceof Reader) {
			((Reader) o).close();
			return true;
		}
		if (o instanceof ServerSocket) {
			((ServerSocket) o).close();
			return true;
		}
		if (o instanceof ZipFile) {
			((ZipFile) o).close();
			return true;
		}
	} catch (IOException e1) {
	}
	return false;
}
/**
 * returns a runtime-exception for deprecated functions
 */
public final static RuntimeException deprecatedException() {
	return new RuntimeException("Function is deprecated");
}
/**
 * returns a dummy enumeration used for deprecated functions
 * @return Enumeration
 */
public final static Enumeration emptyEnumeration() {
	return new Enumeration() {
		public Object nextElement() {
			return null;
		}
		public boolean hasMoreElements() {
			return false;
		}
	};
}
/**
 * flushes an outputstream
 */
public final static void flush(Object o) {
	if (o == null)
		return;
	//
	try {
		if (o instanceof OutputStream) {
			((OutputStream) o).flush();
			return;
		}
		if (o instanceof Writer) {
			((Writer) o).flush();
			return;
		}
	} catch (IOException e1) {
	}
}
/**
 * tries to determine the directory part of a string.
 * returns a string where the last char is a File.separator
 */
public final static String getDirectory(String strFile) {
	//
	if (strFile.length() == 0)
		return strFile;
	//
	strFile = strFile.replace('\\', File.separatorChar);
	strFile = strFile.replace('/', File.separatorChar);
	//
	File f = null;
	// we try it twice (we may remove the file on the first run)
	for (int i = 0; i < 2; i++) {
		if (!strFile.endsWith(File.separator))
			strFile = strFile + File.separator;
		f = new File(strFile);
		if (f.isDirectory())
			return strFile;
		// try to remove the FILE part (strip the separator first)
		strFile = strFile.substring(0, strFile.length() - 1);
		int pp = strFile.lastIndexOf(File.separator);
		if (pp == -1)
			return ""; // strFile was a file w/o path
		strFile = strFile.substring(0, pp);
	}
	return null; // give up, misspelled directory
}
/**
 * translates a string like "User-Agent" to "USER_AGENT"
 * needed for getAttribute() and others
 */
public final static String headerkey2cgikey(String strIn) {
	strIn = strIn.toUpperCase();
	strIn = strIn.replace('-', '_');
	return strIn;
}
/**
 * Parses stuff like "de-de,en-us;something=..."
 */
public final static Enumeration listEnumerator(String s) {
	if (s == null || s.length() == 0)
		return Helper.emptyEnumeration();
	int pp = s.indexOf(";");
	if (pp != -1)
		s = s.substring(0, pp);
	StringTokenizer strt = new StringTokenizer(s, ",");
	Vector v = new Vector();
	while (strt.hasMoreElements()) {
		v.addElement(strt.nextToken().trim());
	}
	return v.elements();
}
/**
 * creates a platform dependent path
 */
public final static String platformPath(String strPath) {
	if (strPath == null)
		return null;
	strPath = strPath.replace('\\', File.separatorChar);
	strPath = strPath.replace('/', File.separatorChar);
	return strPath;
}
/**
 * bubblesorts a string-vector
 */
public static final void sortVector(Vector v) {
	boolean blnHasError = false;
	while (true) {
		blnHasError = false;
		for (int i = 0; i < v.size() - 1; i++) {
			String s1 = (String) v.elementAt(i);
			String s2 = (String) v.elementAt(i + 1);
			if (s1.compareTo(s2) > 0) {
				v.setElementAt(s2, i);
				v.setElementAt(s1, i + 1);
				blnHasError = true;
			}
		}
		if (!blnHasError)
			break;
	}
}
/**
 * Returns an enumeration over the string array
 */
public final static Enumeration stringEnumerator(String s[]) {
	if (s == null)
		return emptyEnumeration();
	//
	Vector v = new Vector(s.length);
	for (int i = 0; i < s.length; i++) {
		v.addElement(s[i]);
	}
	return v.elements();
}
/**
* Converts all of the characters in the string to lower case which are in the range of 'A'-'Z'.  Avoids trouble with turkish locale.
* @param str the string to convert
* @return returns a new string containing the converted characters
*/
public static final String toLowerCase(String str) {
	char[] ch = str.toCharArray();
	toLowerCase(ch);
	return new String(ch);
}
/**
* Converts all of the characters in the string to lower case which are in the range of 'A'-'Z'.  Avoids trouble with turkish locale.
* @param chars the text to convert
*/
public static final void toLowerCase(char[] chars) {
	int len = chars.length;
	//	
	for (int i = 0; i < len; ++i) {
		char ch = chars[i];
		if (ch >= 'A' && ch <= 'Z') {
			chars[i] += CASEDIFF;
		}
	}
}
/**
 * returns a runtime-exception for not-yet-implemented functions
 */
public final static RuntimeException unsupportedException() {
	return new RuntimeException("Function not supported (yet)");
}
}