package net.conacom.servletrunner.util;

/**
 * Copyright 1999-2002 by conacom GmbH
 * Wilhelmshoeher Strasse 202, 60389 Frankfurt/Main, Germany
 * All rights reserved.
 * 
 * This software or implementation is free for commercial and non-commercial use.
 * No warranty is given about the functionality.
 */

/**
 * implementation of wait/notify where lo_wait() respects a lo_notify() which was 
 * sent before lo_wait() was reached.
 * the stupid method names came from the fact that Object declared .wait(..) and .notify() as final.
 *
 * 15.06.02 ho: creation
 */
public class LockObject extends Application {
	// we got a notify so we don't need to .wait()
	private volatile boolean blnGotNotify = false;
/**
 *
 */
public LockObject() {
	super();
}
/**
 * 
 */
public void dispose() {
	if (blnDisposed)
		return;
	blnDisposed = true;
	lo_reset();
}
/**
 * 
 */
public boolean isDisposed() {
	return blnDisposed;
}
/**
 * notifies lo_wait(..)
 */
public void lo_notify() {
	synchronized (this) {
		blnGotNotify = true;
		super.notifyAll();
	}
}
/**
 * clears the state
 */
public void lo_reset() {
	blnGotNotify = false;
}
/**
 * wait forever on lo_notify();
 */
public void lo_wait() {
	lo_wait(0);
}
/**
 * wait lngMS miliseconds on a lo_notify()
 */
public void lo_wait(long lngMS) {
	synchronized (this) {
		if (!blnGotNotify) {
			try {
				super.wait(lngMS);
			} catch (InterruptedException e) {
			}
		}
		blnGotNotify = false; // clear flag
	}
}
}